// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik
// Universitaet Duisburg, Germany
// Institute for Combustion and Gas Dynamics
// University of Duisburg, Germany
// Sat Aug 21 1999
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!
#ifndef ArtCompressibleBViscFlux2D_HH
#define ArtCompressibleBViscFlux2D_HH

#include <mouse_config.h>

class ArtCompressibleBViscFlux2D;

#include <mouse_types.hh>
#include <ConstructArg.hh>
#include <ArtCompressibleViscFlux2D.hh>

/**
*/
class ArtCompressibleBViscFlux2D : public IncompressibleOperator2D
{
protected:
  /** */ real Re_n_inv;
  /** */ VarContainer::var1_t du_dx;
  /** */ VarContainer::var1_t du_dy;
  /** */ VarContainer::var1_t dv_dx;
  /** */ VarContainer::var1_t dv_dy;

public:
  /** */ ArtCompressibleBViscFlux2D (carg_t arg);
  /** */ virtual ~ArtCompressibleBViscFlux2D ();
  /** */ void operate(size_t ie, size_t ip, size_t from, size_t to, 
		      real nx, real ny, bool mod_from, bool mod_to);
};

inline void ArtCompressibleBViscFlux2D::operate(size_t ie, size_t ip, 
						size_t from, size_t to, 
						real nx, real ny, 
						bool mod_from, bool mod_to)
{
  real flux_u_from = Re_n_inv * (du_dx[from] * nx + du_dy[from] * ny);
  real flux_v_from = Re_n_inv * (dv_dx[from] * nx + dv_dy[from] * ny);
  real flux_u_to = Re_n_inv * (du_dx[to] * nx + du_dy[to] * ny);
  real flux_v_to = Re_n_inv * (dv_dx[to] * nx + dv_dy[to] * ny);
  
  // Add to raw residual (negative !)
  if (mod_from) {
    res_u[from] -= flux_u_from;
    res_v[from] -= flux_v_from;
  };
  if (mod_to) {
    res_u[to] -= flux_u_to;
    res_v[to] -= flux_v_to;
  };
};


#endif
