// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#include "BoundaryPointList.hh"

BoundaryPointList::BoundaryPointList(ConstructArg arg) : MList(arg)
{
  AddClassName();
  PL = FindObject<PointList>(false, true);
  code = BuildObject<MIntList>(arg);
  MIntList *read_prio = BuildObject<MIntList>(arg);
  if (code->NumEntries() != read_prio->NumEntries()) {
    string text = "";
    text += code->NumEntries();
    text += " codes, but only ";
    text += read_prio->NumEntries();
    text += " priorieties specified";
    Error(text);
  };
  FORALL(i, code->) {
    if ((code->At(i) < -999) && (code->At(i) > 999)) {
      Error("currently only codes between -999 and +999 are allowed");
    };
  };
  for (uint i = 0; i < 2000; i++) {
    prio[i] = 0;
    code_points[i] = NULL;
  };
  FORALL(i, code->) {
    prio[code->At(i) + 999] = read_prio->At(i);
    code_points[code->At(i) +999] = new TList<size_t>(10, 10);
  };
  bindex = new TList<size_t>(PL);
  findex = new TList<size_t>(this);
  nx = new TList<real>(this, 0);
  ny = new TList<real>(this, 0);
  nz = new TList<real>(this, 0);
  Update();
  delete read_prio;
};

BoundaryPointList::~BoundaryPointList()
{
  for (uint i = 0; i < 2000; i++) delete code_points[i];
  delete findex;
  delete bindex;
  delete nx;
  delete ny;
  delete nz;
  delete code;
};

void BoundaryPointList::AddPoint(size_t point)
{
  bindex->At(point) = findex->AddEntry();
  findex->At(bindex->At(point)) = point;
  if (code_points[PL->code(point) + 999] == NULL) {
    string text = "undefined boundary code ";
    text += PL->code(point);
    text += " found";
    Error(text);
  };
  code_points[PL->code(point) + 999]->NewEntry() = point;
};

void BoundaryPointList::Update()
{
  DelAll();
  for (uint i = 0; i < 2000; i++) {
    if (code_points[i] != NULL) code_points[i]->DelAll();
  };
  FORALL(i, PL->) {
    if (PL->code(i) != 0) {
      AddPoint(i);
    };
  };
};












