// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#if !defined(BOUNDARYPOINTLIST_HH)
#define BOUNDARYPOINTLIST_HH

class BoundaryPointList;

#include <MList.hh>
#include <ConstructArg.hh>
#include <PointList.hh>
#include <TList.hh>

/** additional information about boundary points.
    \\
    {\bf construtor parameters:}
    \begin{itemize}
    \item ...
    \item ...
    \end{itemize}
*/
class BoundaryPointList : public MList
{
  class iterator;
  friend class BoundaryPointList::iterator;

protected:
  /** */ int prio[2000];
  /** */ TList<size_t> *code_points[2000];
  /** */ TList<size_t> *bindex;
  /** */ TList<size_t> *findex;
  /** */ TList<real> *nx;
  /** */ TList<real> *ny;
  /** */ TList<real> *nz;
  /** */ MIntList *code;
  /** */ PointList *PL;

public:
  /** */ BoundaryPointList(ConstructArg arg);
  /** */ ~BoundaryPointList();
  /** */ size_t BIndex(size_t i) { return bindex->At(i); };
  /** */ size_t FIndex(size_t i) { return findex->At(i); };
  /** */ size_t FIndexCode(int c, size_t i) { return code_points[c + 999]->At(i); };
  /** */ real Nx(size_t i) { return nx->At(BIndex(i)); };
  /** */ real Ny(size_t i) { return ny->At(BIndex(i)); };
  /** */ real Nz(size_t i) { return nz->At(BIndex(i)); };
  /** */ int Prio(size_t i) { return prio[PL->code(i) + 999]; };
  /** */ void SetNx(size_t i, real a) { nx->At(BIndex(i)) = a; };
  /** */ void SetNy(size_t i, real a) { ny->At(BIndex(i)) = a; };
  /** */ void SetNz(size_t i, real a) { nz->At(BIndex(i)) = a; };
  /** */ void AddPoint(size_t point);
  /** */ void Update();
  /** */ //TList<size_t>* CodePoints(int a_code) { return code_points[a_code + 999]; };
  /** */ size_t NumCodes() { return code->NumEntries(); };
  /** */ int Code(size_t i) { return code->At(i); };
  /** */ size_t Point(int c, size_t i) { return code_points[c + 999]->At(i); };
  /** */ size_t begin(int c) { return code_points[c + 999]->begin(); };
  /** */ size_t end(int c) { return code_points[c + 999]->end(); };
  /** */ size_t next(int c, size_t i) { return code_points[c + 999]->next(i); };
  /** */ size_t begin() { return List::begin(); };
  /** */ size_t end() { return List::end(); };
  /** */ size_t next(size_t i) { return List::next(i); };
};

#endif












