// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#include "CflManager.hh"
#include <TimeStepWidthOperator.hh>
#include <TPointCalculator.hh>

CflManager::CflManager(carg_t arg) : POperation(arg)
{
  AddClassName();
  Init();
  delta_t = new TList<real>(PL);
  cfl = arg.P->ReadReal();
  string mode;
  mode = arg.P->ReadString();
  bool mode_found = false;
  if (mode == "steady") {
    unsteady = false;
    mode_found = true;
  };
  if (mode == "unsteady") {
    unsteady = true;
    mode_found = true;
  };
  if (!mode_found) Error("please specify 'steady' or 'unsteady'");
  for (size_t i = 0; i < num_threads; i++) {
    current_thread = i;
    arg.P->Reset();
    cfl = arg.P->ReadReal();
    mode = arg.P->ReadString();
    calc->At(i) = BuildObject<PointCalculator>(arg);
  };
};

void CflManager::Act()
{
  POperation::Act();
  if (unsteady) {
    global_delta_t = 1e10;
    FORALL(i, delta_t->) {
      if (delta_t->At(i) < global_delta_t) {
	global_delta_t = delta_t->At(i);
      };
    };
    FORALL(i, delta_t->) {
      delta_t->At(i) = global_delta_t;
    };
  };
};








