// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#ifndef CompressibleBFlux2D_HH
#define CompressibleBFlux2D_HH

class CompressibleBFlux2D;

#include <CompressibleOperator2D.hh>

/** */
class CompressibleBFlux2D : public CompressibleOperator2D
{
public:
  /** */ CompressibleBFlux2D(carg_t arg);
  /** */ void operate(size_t ie, size_t ip, size_t from, size_t to, 
		      real nx, real ny, bool mod_from, bool mod_to);
};

inline void CompressibleBFlux2D::operate(size_t ie, size_t ip, 
					 size_t from, size_t to, 
					 real nx, real ny, 
					 bool mod_from, bool mod_to)
{
  //cout << "CompressibleBFlux2D::operate  " << from << " -> " << to << endl;
  {
    //.. for the triangles adjacent to node "..from"
    //.... average on adjacent half edge, side "..from"
    real rho_h = rho[from];
    real rhou_h = rhou[from];
    real rhov_h = rhov[from];
    real rhoE_h = rhoE[from];
    //.... some usefull quantities
    real irho_h = 1.0 / rho_h;
    real rho_q_u = rhou_h * rhou_h * irho_h;
    real rho_q_v = rhov_h * rhov_h * irho_h;
    real rho_uv = rhou_h * rhov_h * irho_h;
    real p = gamma1 * (rhoE_h - 0.5 * (rho_q_u + rho_q_v));
    real h = (rhoE_h + p) * irho_h;
    //.... compute (h*n)*da for the cell interface
    real flux_rho = rhou_h * nx + rhov_h * ny;
    real flux_rhou = (rho_q_u + p) * nx + rho_uv * ny;
    real flux_rhov = rho_uv * nx + (rho_q_v + p) * ny;
    real flux_rhoE = h * flux_rho;
    //.... Add to pointwise residual for "..from"
    if (mod_from) {
      res_rho[from] += flux_rho;
      res_rhou[from] += flux_rhou;
      res_rhov[from] += flux_rhov;
      res_rhoE[from] += flux_rhoE;
    };
  };
  {
    //.. for the triangles adjacent to node "..to"
    //.... average on adjacent half edge, side "..to"
    real rho_h = rho[to];
    real rhou_h = rhou[to];
    real rhov_h = rhov[to];
    real rhoE_h = rhoE[to];
    //.... some usefull quantities
    real irho_h = 1.0 / rho_h;
    real rho_q_u = rhou_h * rhou_h * irho_h;
    real rho_q_v = rhov_h * rhov_h * irho_h;
    real rho_uv = rhou_h * rhov_h * irho_h;
    real p = gamma1 * (rhoE_h - 0.5 * (rho_q_u + rho_q_v));
    real h = (rhoE_h + p) * irho_h;
    //.... compute (h*n)*da for the cell interface
    real flux_rho = rhou_h * nx + rhov_h * ny;
    real flux_rhou = (rho_q_u + p) * nx + rho_uv * ny;
    real flux_rhov = rho_uv * nx + (rho_q_v + p) * ny;
    real flux_rhoE = h * flux_rho;
    //.... Add to pointwise residual for "..to"
    if (mod_to) {
      res_rho[to] += flux_rho;
      res_rhou[to] += flux_rhou;
      res_rhov[to] += flux_rhov;
      res_rhoE[to] += flux_rhoE;
    };
  };
};

#endif





