// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#ifndef CompressibleOperator2D_HH
#define CompressibleOperator2D_HH

class CompressibleOperator2D;

#include <Operator.hh>
#include <Parameter.hh>

/**
   base class for compressible gas flow operators in two dimensions.
*/
class CompressibleOperator2D : public Operator
{
protected:
  /** free stream Mach number */ 
  real Mach_oo;

  /** angle of attack */ 
  real alpha;

  /** specific heat ratio */ 
  real gamma;

  /** free stream density */ 
  real rho_oo;

  /** free stream x momentum */
  real rhou_oo;

  /** free stream y momentum */
  real rhov_oo;

  /** free stream energy */
  real rhoE_oo;

  /** free stream pressure */
  real p_oo;

  /** a useful coefficient */
  real gamma1;

  /** a useful coefficient */
  real gamma2;

  /** a useful coefficient */
  real igamma1;

  /** object holding some parameters */
  Parameter *PARA;

  /** the name of the variable field used for computations */
  string new_field_name;

  /** the name of the residual field */
  string res_field_name;

  /** the density field */
  VarContainer::var1_t rho;

  /** the x momentum field */
  VarContainer::var1_t rhou;

  /** the y momentum field */
  VarContainer::var1_t rhov;

  /** the energy field */
  VarContainer::var1_t rhoE;

  /** the density residual field */
  VarContainer::var1_t res_rho;

  /** the x momentum residual field */
  VarContainer::var1_t res_rhou;

  /** the y momentum residual field */
  VarContainer::var1_t res_rhov;

  /** the energy residual field */
  VarContainer::var1_t res_rhoE;

public:
  /** */ CompressibleOperator2D(carg_t arg);
};



#endif








