// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#ifndef CompressibleTStep2D_HH
#define CompressibleTStep2D_HH

class CompressibleTStep2D;

#include <CompressibleOperator2D.hh>
#include <CflManager.hh>

/** */
class CompressibleTStep2D : public CompressibleOperator2D
{
protected:
  /** */ CflManager *CFL;
public:
  /** */ CompressibleTStep2D(carg_t arg);
  /** */ void operate(size_t it, size_t i);
};

inline void CompressibleTStep2D::operate(size_t it, size_t i)
{
  real q_rho_uv = rhou[i]*rhou[i] + rhov[i]*rhov[i];
  real speed = sqrt(q_rho_uv) / rho[i];
  real a_speed = ((rhoE[i] * rho[i] - .5 * q_rho_uv) * gamma2) / (rho[i]*rho[i]);
  // avoid division by 0 if critical
  if(a_speed < 0.1) a_speed=0.1;    
  a_speed = sqrt(a_speed);
  CFL->SetTimeStep(i, PL->ChDist(i) / (speed + a_speed));
};

#endif





