// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#ifndef CompressibleWFlux2D_HH
#define CompressibleWFlux2D_HH

class CompressibleWFlux2D;

#include <CompressibleOperator2D.hh>

/** */
class CompressibleWFlux2D : public CompressibleOperator2D
{
public:
  /** */ CompressibleWFlux2D(carg_t arg);
  /** */ void operate(size_t ie, size_t ip, size_t from, size_t to, 
		      real nx, real ny, bool mod_from, bool mod_to);
};

inline void CompressibleWFlux2D::operate(size_t ie, size_t ip, 
					 size_t from, size_t to, 
					 real nx, real ny, 
					 bool mod_from, bool mod_to)
{
  if (mod_from) {
    /*
    real rho_h = .75*rho[from]+.25*rho[to];
    real rhou_h = .75*rhou[from]+.25*rhou[to];
    real rhov_h = .75*rhov[from]+.25*rhov[to];
    real rhoE_h = .75*rhoE[from]+.25*rhoE[to];
    */
    real rho_h = rho[from];
    real rhou_h = rhou[from];
    real rhov_h = rhov[from];
    real rhoE_h = rhoE[from];
    real p = gamma1 * (rhoE_h - 0.5 * (rhou_h * rhou_h + rhov_h * rhov_h) / rho_h);
    res_rhou[from] += p * nx;
    res_rhov[from] += p * ny;
  };
  if (mod_to) {
    /*
    real rho_h = .75*rho[to]+.25*rho[from];
    real rhou_h = .75*rho[to]+.25*rhou[from];
    real rhov_h = .75*rho[to]+.25*rhov[from];
    real rhoE_h = .75*rho[to]+.25*rhoE[from];
    */
    real rho_h = rho[to];
    real rhou_h = rhou[to];
    real rhov_h = rhov[to];
    real rhoE_h = rhoE[to];
    real p = gamma1 * (rhoE_h - 0.5 * (rhou_h * rhou_h + rhov_h * rhov_h) / rho_h);
    res_rhou[to] += p * nx;
    res_rhov[to] += p * ny;
  };
};

#endif












