// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik
// Universitaet Duisburg, Germany
// Institute for Combustion and Gas Dynamics
// University of Duisburg, Germany
// Mon May 17 1999
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#include "ComputeMaxRes.hh"

ComputeMaxRes::ComputeMaxRes(carg_t arg)
  : Action(arg)
{
  AddClassName();
  VL = FindObject<VarList>(false, true);
  PL = FindObject<PointList>(false, true);
  string name1 = arg.P->ReadString();
  string name2 = arg.P->ReadString();
  var1 = VL->Get2DVar(name1);
  var2 = VL->Get2DVar(name2);
  max_value = new real[VL->NumVars()];
  l2_value = new real[VL->NumVars()];
  mga = LookForObject<MultiGridAgent<PopList2D> >(false, true);
};

ComputeMaxRes::~ComputeMaxRes()
{
};

void ComputeMaxRes::Act()
{
  bool finest = (mga == NULL);
  if (!finest) finest = (mga == mga->Finest());
  for (size_t iv = 0; iv < VL->NumVars(); iv++) {
    max_value[iv] = 0;
    l2_value[iv] = 0;
  };
  size_t N = 0;
  FORALL(i, VL->) {
    bool comp = finest;
    if (!comp) comp = (PL->code(i) == 0);
    if (comp) {
      for (size_t iv = 0; iv < VL->NumVars(); iv++) {
	real diff = fabs(var1[iv][i] - var2[iv][i]);
	if (diff > max_value[iv]) max_value[iv] = diff;
	l2_value[iv] += diff*diff;
      };
    };
    N++;
  };
  for (size_t iv = 0; iv < VL->NumVars(); iv++) l2_value[iv] = sqrt(l2_value[iv]/N);
};
