// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik
// Universitaet Duisburg, Germany
// Institute for Combustion and Gas Dynamics
// University of Duisburg, Germany
// Mon May 17 1999
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!
#ifndef CorrectionDamper2D_HH
#define CorrectionDamper2D_HH

#include <mouse_config.h>

class CorrectionDamper2D;

#include <mouse_types.hh>
#include <ConstructArg.hh>
#include <Operator.hh>

/**
*/
class CorrectionDamper2D : public Operator
{
protected:
  /** */ real K;
  /** */ VarContainer::var2_t gx;
  /** */ VarContainer::var2_t gy;
  /** */ VarContainer::var2_t res;
  /** */ VarContainer::var2_t damp_const;

public:
  /** */ CorrectionDamper2D (carg_t arg);
  /** */ virtual ~CorrectionDamper2D ();
  /** */ void operate(size_t ie, size_t ip, size_t from, size_t to, 
		      real nx, real ny, bool mod_from, bool mod_to);
};

inline void CorrectionDamper2D::operate(size_t ie, size_t ip, size_t from, size_t to, 
					real nx, real ny, bool mod_from, bool mod_to)
{
  real flux;
  for (size_t iv = 0; iv < VL->NumVars(); iv++) {
    flux = -0.5*K*((gx[iv][from] + gx[iv][to])*nx + (gy[iv][from] + gy[iv][to])*ny);
    //flux *= -.5*K*(damp_const[iv][from] + damp_const[iv][to]);
    if (mod_from) res[iv][from] -= flux;
    if (mod_to) res[iv][to] += flux;
  };
};


#endif
