// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#if !defined(CppParser_HH)
#define CppParser_HH

class CppParser;

#include <Parser.hh>
#include <TList.hh>

class CppParser : public Parser
{

  struct pitem_t 
  {
    virtual void dummy() {};
    virtual ~pitem_t() {};
  };

  template<class T>
  struct tpitem_t : public pitem_t
  {
    T data;
    tpitem_t(T a_data) { data = a_data; };
  };

protected:
  /** */ TList<pitem_t*> *data_list;
  /** */ size_t current_item;

public:
  /** */ CppParser(MObject *a_parent);
  /** */ ~CppParser();
  /** */ template<class T> void Add(T a_data);
  /** */ template<class T> T Read();
  /** */ template<class T> void Read(T &value) { value = Read<T>(); };
  /** */ virtual int ReadInt() { return Read<int>(); };
  /** */ virtual size_t ReadIndex() { return Read<size_t>(); };
  /** */ virtual string ReadString() { return Read<string>(); };
  /** */ virtual real ReadReal() { return Read<real>(); };
  /** */ virtual void Reset() { current_item = data_list->begin(); };
  /** */ virtual MObject* ResolveVar() { return Read<MObject*>(); };
  /** */ virtual Parser* SubParser();
  /** */ virtual bool eof() { return current_item >= data_list->end(); };
};

template<class T>
void CppParser::Add(T a_data)
{
  data_list->NewEntry() = new tpitem_t<T>(a_data);
};

template<class T>
T CppParser::Read()
{
  pitem_t *item = data_list->At(current_item);
  current_item = data_list->next(current_item);
  tpitem_t<T> *titem = dynamic_cast<tpitem_t<T>*>(item);
  if (titem == NULL) Error("error while reading from parser");
  return titem->data;
};

#endif







