// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#include "Delaunay2D.hh"
#include <geometry2D.hh>

//
//.. constructor
//
Delaunay2D::Delaunay2D(ConstructArg arg) : Mesh2DAction(arg)
{
  AddClassName();
  EL = FindObject<ElementList>(false, true);
  PEL = FindObject<PelList2D>(false, true);
  PL = FindObject<PointList>(false, true);
};

//
//.. DelaunaySwitch
//
bool Delaunay2D::DelaunaySwitch(size_t p_from, size_t p_to,
				size_t tri_left, size_t tri_right)
{
  real t1, t2, t4, t5, t14, t17, t18, t20, t22, t26, t27, t29, t31;
  real t41, t42, t50, t52, t57, t59, t61, t71, t73, t94, t103;
  real x0, y0, x1, x2, x3, x4, y1, y2, y3, y4, a11, a12, a22;
  size_t p_left, p_right, e_new;
  int n;
  n = MESH->FindNode(tri_left, p_to);
  p_left = EL->PermNode(tri_left, n+1);
  n = MESH->FindNode(tri_right, p_from);
  p_right = EL->PermNode(tri_right, n+1);

  real k, l;
  Intersection(k, l, 
	       PL->x(p_from), PL->y(p_from),
	       PL->x(p_to) - PL->x(p_from), 
	       PL->y(p_to) - PL->y(p_from),
	       PL->x(p_left), PL->y(p_left),
	       PL->x(p_right) - PL->x(p_left), 
	       PL->y(p_right) - PL->y(p_left));
  //cout << "k:" << k << "  l:" << l << endl;
  if ((k < 1e-9) || (k > 1-1e-9)) return false;

  // compute coordinates
  x1 = PL->x(p_from);
  y1 = PL->y(p_from);
  x2 = PL->x(p_right);
  y2 = PL->y(p_right);
  x3 = PL->x(p_to);
  y3 = PL->y(p_to);
  x4 = PL->x(p_left);
  y4 = PL->y(p_left);
  x0 = .25*(x1 + x2 + x3 + x4);
  y0 = .25*(y1 + y2 + y3 + y4);
  a11 = .25*(MESH->A11(p_from) + MESH->A11(p_right) 
	     + MESH->A11(p_to) + MESH->A11(p_left));
  a12 = .25*(MESH->A12(p_from) + MESH->A12(p_right) 
	     + MESH->A12(p_to) + MESH->A12(p_left));
  a22 = .25*(MESH->A22(p_from) + MESH->A22(p_right) 
	     + MESH->A22(p_to) + MESH->A22(p_left));
  MESH->Transform(x0, y0, a11, a12, a22, x1, y1);
  MESH->Transform(x0, y0, a11, a12, a22, x2, y2);
  MESH->Transform(x0, y0, a11, a12, a22, x3, y3);
  MESH->Transform(x0, y0, a11, a12, a22, x4, y4);
  
  // beginning of MAPLE-code
  t1 = y2*x1;
  t2 = x4*x4;
  t4 = y2*y2;
  t5 = x4*t4;
  t14 = y1*y1;
  t17 = x4*y2;
  t18 = y3*y3;
  t20 = x3*x3;
  t22 = y4*y4;
  t26 = x1*x1;
  t27 = y2*t26;
  t29 = -t1*t2-y1*t5+y4*t4*x1+y3*t5-x3*y4*t4+x3*t2*y2-x3*t14*y2-t18*t17-t20
    *t17+x3*t22*y2+t14*t17-x3*t27;
  t31 = x2*t26;
  t41 = x1*y3;
  t42 = x2*x2;
  t50 = x4*t27-y4*t31-y3*x4*t26+y3*t31+x3*y4*t26+t20*t1-t22*t1+t18*t1-t42*
    t41-y4*t18*x1-y4*t42*x3+y4*t20*x2;
  t52 = x2*y1;
  t57 = x2*t2;
  t59 = y3*x4;
  t61 = y1*x4;
  t71 = x2*t22;
  t73 = -t18*t52+y3*t14*x2-t20*t52+y1*t57-t14*t59+t18*t61-t42*t61-x3*y1*t2-
    y3*t57+x3*y1*t42+y4*t18*x2-y3*t71;
  t94 = -x3*y1*t22+y1*t71-t14*x2*y4+x3*t14*y4+t20*t61-y4*t20*x1+y3*t2*x1+y4
    *t42*x1+t42*t59+y3*t22*x1+x3*y1*t4-t4*t41;
  t103 = -1/(x2*y3-t52-t41-y2*x3+t1+y1*x3)*(t29+t50+t73+t94);
  // end of MAPLE-code
  
  if (t103 < 0) { // switch diagonal
    PEL->DeleteFrom(tri_left);
    EL->DelEntry(tri_left);
    PEL->DeleteFrom(tri_right);
    EL->DelEntry(tri_right);
    e_new = EL->AddElement(3, p_from, p_right, p_left);
    PEL->AddTo(p_from, e_new);
    PEL->AddTo(p_right, e_new);
    PEL->AddTo(p_left, e_new);
    e_new = EL->AddElement(3, p_to, p_left, p_right);
    PEL->AddTo(p_to, e_new);
    PEL->AddTo(p_left, e_new);
    PEL->AddTo(p_right, e_new);
    /*
    e_new = EL->AddElement(_QUAD, p_from, p_right, p_to, p_left);
    PEL->AddTo(p_to, e_new);
    PEL->AddTo(p_left, e_new);
    PEL->AddTo(p_right, e_new);
    PEL->AddTo(p_from, e_new);
    cout << "switched!" << endl;
    */
    return true;
  };
  return false;
};

//
//.. Act
//
void Delaunay2D::Act()
{
  size_t num_reorg = 0, old_nr;
  size_t i, j, ne, p1, p2, e1, e2;
  bool fe1, fe2;
  do {
    ne = EL->NumEntries();
    old_nr = num_reorg;
    num_reorg = 0;
    for (i = EL->First(); i < ne; i = EL->Next(i)) {
      if (MESH->ElementMarked(i)) {
	for (j = 0; j < EL->NumNodes(i); j++) {
	  p1 = EL->PermNode(i, j);
	  p2 = EL->PermNode(i, j + 1);
	  MESH->FindElements(p1, p2, e1, e2, fe1, fe2);
	  if (fe1 && fe2) {
	    if ((EL->NumNodes(e1) == 3) && (EL->NumNodes(e2) == 3)) {
	      if (DelaunaySwitch(p1, p2, e1, e2)) {
		num_reorg++;
		break;
	      };
	    };
	  };
	};
      };
    };
    cout << num_reorg << " triangle-reconfigurations" << endl;
  } while (num_reorg < old_nr);
};		







