// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#include "DrawCircle.hh"
#include <math.h>

//
//.. constructor
//
DrawCircle::DrawCircle(ConstructArg arg) : Mesh2DGeometryAction(arg)
{
  AddClassName();
  xm = arg.P->ReadReal();
  ym = arg.P->ReadReal();
  r = arg.P->ReadReal();
  steps = arg.P->ReadInt();
  dir = arg.P->ReadReal();
  code = arg.P->ReadInt();
  if (dir < 0) dir = -1;
  else dir = 1;
};

//
//.. Act
//
void DrawCircle::Act() {
  real alpha = 0;
  real d_alpha = 2 * M_PI / steps;
  size_t p0 = PL->AddEntry();
  MESH->FixPoint(p0);
  MESH->SetMatrixComp(p0, 0, 0, 0);
  MESH->SetNetDensity(p0, 0);
  size_t p1=0, p2=0, i=0;
  PL->x(p0) = xm + r*sin(alpha);
  PL->y(p0) = ym + r*cos(alpha);
  PL->code(p0) = code;
  p1 = p0;
  for (i = 1; i < steps; i++) {
    alpha = i * dir * d_alpha;
    p2 = PL->AddEntry();
    MESH->FixPoint(p2);
    MESH->SetMatrixComp(p2, 0, 0, 0);
    MESH->SetNetDensity(p2, 0);
    PL->x(p2) = xm + r*sin(alpha);
    PL->y(p2) = ym + r*cos(alpha);
    PL->code(p2) = code;
    TRI->AddFront(p1, p2);
    p1 = p2;
  };
  TRI->AddFront(p2, p0);  
};



















