// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#include "EventChannel.hh"
#include "ConstructArg.hh"

//
//.. constructor
//
EventChannel::EventChannel (ConstructArg arg, 
			    string host, 
			    unsigned short int port) : 
  MObject (arg)
{
  next_channel = NULL;
  remote_host = host;
  remote_port = port;
  event_pending = false;
};

//
//.. Handshake
//
void EventChannel::Handshake()
{
  if (!Connect(0)) Error("handshake failed"); // handshake
  string status = ancestor->ReadString(sock);
  if (status != "ok") Error("handshake failed");
  close(sock);
};

//
//.. CreateSocket
//
bool EventChannel::Connect(char con_type)
{
  struct sockaddr_in sa;
  memset(&sa, 0, sizeof(struct sockaddr_in));
  struct hostent *hp;
  hp = gethostbyname(remote_host.c_str());
  if (hp == NULL) Error ("couldn't connect to host");
  memcpy((char*) &sa.sin_addr, hp->h_addr_list[0], hp->h_length);
  sa.sin_family = AF_INET;
  sa.sin_port = htons(remote_port);
  sock = socket(AF_INET, SOCK_STREAM, 0);
  if (sock < 0) Error("couldn't create socket");
  ///cout << "..." << endl;
  if (connect(sock, (sockaddr*) &sa, sizeof(struct sockaddr_in)) < 0) {
    close(sock);
    return false;
  } else {
    write(sock, &con_type, 1);
    ancestor->WriteString(sock, ancestor->host);
    ancestor->WriteInt(sock, ancestor->port_number);
  };
  return true;
};

//
//.. EventAvailable
//
bool EventChannel::EventAvailable()
{
  if (event_pending) {
    event_pending = false;
    return true;
  };
  return false;
};

//
//.. Send
//
void EventChannel::Send(Event &E)
{
  //cout << "sending ";
  //E.Print();

  ancestor->WriteInt(sock, E.num_reactions);
  ancestor->WriteInt(sock, E.invalid);
  ancestor->WriteInt(sock, E.done);
  ancestor->WriteInt(sock, E.single_recipient);
  ancestor->WriteInt(sock, E.broadcast);
  ancestor->WriteInt(sock, E.send_to_other_nodes);
  ancestor->WriteInt(sock, E.send_only_up);
  ancestor->WriteInt(sock, E.send_only_down);
  ancestor->WriteInt(sock, E.only_same_mesh);
  ancestor->WriteString(sock, E.name);
  ancestor->WriteString(sock, E.recipient_id);
  ancestor->WriteString(sock, E.sender_id);
  ancestor->WriteString(sock, E.sender_mesh_id);
  
  size_t i;
  ancestor->WriteInt(sock, E.NumOutIndices());
  for (i = 0; i < E.NumOutIndices(); i++) ancestor->WriteInt(sock, E.OutInt(i));
  ancestor->WriteInt(sock, E.NumOutReals());
  for (i = 0; i < E.NumOutReals(); i++) ancestor->WriteReal(sock, E.OutReal(i));
  ancestor->WriteInt(sock, E.NumOutStrings());
  for (i = 0; i < E.NumOutStrings(); i++) ancestor->WriteString(sock, E.OutString(i));

  ancestor->WriteInt(sock, E.NumInIndices());
  for (i = 0; i < E.NumInIndices(); i++) ancestor->WriteInt(sock, E.InInt(i));
  ancestor->WriteInt(sock, E.NumInReals());
  for (i = 0; i < E.NumInReals(); i++) ancestor->WriteReal(sock, E.InReal(i));
  ancestor->WriteInt(sock, E.NumInStrings());
  for (i = 0; i < E.NumInStrings(); i++) ancestor->WriteString(sock, E.InString(i));

  string status = ancestor->ReadString(sock);
  if (status != "ok") Error("error sending event to remote");
};

//
//.. Send
//
void EventChannel::Receive(Event &E)
{
  E.num_reactions = ancestor->ReadInt(sock);
  E.invalid = ancestor->ReadInt(sock);
  E.done = ancestor->ReadInt(sock);
  E.single_recipient = ancestor->ReadInt(sock);
  E.broadcast = ancestor->ReadInt(sock);
  E.send_to_other_nodes = ancestor->ReadInt(sock);
  E.send_only_up = ancestor->ReadInt(sock);
  E.send_only_down = ancestor->ReadInt(sock);
  E.only_same_mesh = ancestor->ReadInt(sock);
  E.name = ancestor->ReadString(sock);
  E.recipient_id = ancestor->ReadString(sock);
  E.sender_id = ancestor->ReadString(sock);
  E.sender_mesh_id = ancestor->ReadString(sock);

  delete E.in_int;
  E.in_int = new TList<long int>(5, 5);
  delete E.out_int;
  E.out_int = new TList<long int>(5, 5);
  delete E.in_real;
  E.in_real = new TList<real>(5, 5);
  delete E.out_real;
  E.out_real = new TList<real>(5, 5);
  delete E.in_string;
  E.in_string = new TList<string>(5, 5, "");
  delete E.out_string;
  E.out_string = new TList<string>(5, 5, "");
  
  int i, n;
  n = ancestor->ReadInt(sock);
  for (i = 0; i < n; i++) E.AddOutInt(ancestor->ReadInt(sock));
  n = ancestor->ReadInt(sock);
  for (i = 0; i < n; i++) E.AddOutReal(ancestor->ReadReal(sock));
  n = ancestor->ReadInt(sock);
  for (i = 0; i < n; i++) E.AddOutString(ancestor->ReadString(sock));

  n = ancestor->ReadInt(sock);
  for (i = 0; i < n; i++) E.AddInInt(ancestor->ReadInt(sock));
  n = ancestor->ReadInt(sock);
  for (i = 0; i < n; i++) E.AddInReal(ancestor->ReadReal(sock));
  n = ancestor->ReadInt(sock);
  for (i = 0; i < n; i++) E.AddInString(ancestor->ReadString(sock));

  E.origin_channel = this;
  //cout << "received ";
  //E.Print();

  ancestor->WriteString(sock, "ok");
};







