// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#if !defined(FEDGEOPERATOR2D_HH)
#define FEDGEOPERATOR2D_HH

#include <mouse_types.hh>
#include "VarList.hh"

class FEdgeOperator2D;

/**
   FEdgeOperator2D is an abstract definition for 2D edge-wise operations.
   The class FEdgeOperator2D is not intended to be used for inheritance. 
   Instead it defines the functionality, which edge-wise operator
   classes have to provide to use them as template parameters for 
   TFEdgeCalculator2D.
*/
class FEdgeOperator2D
{
public:
  /** This method executes an edge-wise computation for a single edge.
      @param ie the index within the local set of edges, belonging to
      one group (used to access temporary edge data)
      @param ip the index within the local set of points, belonging to
      one group (used to access temporary point data)
      @param from starting point of the edge
      @param to ending point of the edge
      @param nx the x-component of the interface normal vector
      @param ny the y-component of the interface normal vector
      @param mod_from a flag, specifying if data on the from side
      may be modified
      @param mod_to  a flag, specifying if data on the to side
      may be modified
  */
  void operate(size_t ie, size_t ip, size_t from, size_t to, 
	       real nx, real ny, bool mod_from, bool mod_to);
};

#endif











