#include <mouse_config.h>
#ifdef WITH_QT

#include "MQCellInfoPlotter.hh"
#include <FindMinMax.hh>

MQCellInfoPlotter::MQCellInfoPlotter(ConstructArg arg, QWidget *parent,
				     const char *name) 
  : MQPlotter(arg, parent, name)
{
  AddClassName();
  PL = FindObject<PointList>(false, true);
  POPL = FindObject<PopList2D>(false, true);
};

void MQCellInfoPlotter::AddBoundaryVectors(CellProjection2D *fcell, size_t i)
{
  for (size_t j = 0; j < POPL->NumBLinks(i); j++) {
    fcell->AddVector(POPL->Bnx(i,j), POPL->Bny(i,j));
  };
};

void MQCellInfoPlotter::Plot(MQPlotWidget *pw, QPainter *P, const QRect *R)
{
  FORALL(i, PL->) {
    if (pw->IsIn(PL->x(i), PL->y(i))) {

      // set up the projection function
      CellProjection2D fcell;
      for (size_t j = 0; j < POPL->NumFLinks(i); j++) {
	fcell.AddVector(POPL->Fnx(i,j), POPL->Fny(i,j));
      };
      AddBoundaryVectors(&fcell, i);

      // find the max and min values
      FindMinMax<CellProjection2D> FMM(&fcell, 0, M_PI, 20, 0.05);
      real rmin = .5*FMM.Min();
      real rmax = .5*FMM.Max();

      // draw the min and max circles
      P->setBrush(NoBrush);
      P->setPen(red);
      pw->DrawCircle(P, PL->x(i), PL->y(i), rmax);
      P->setPen(green);
      pw->DrawCircle(P, PL->x(i), PL->y(i), rmin);
    };
  };
};

#endif
