// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#include <mouse_config.h>
#ifdef WITH_QT

#include "MQIsolinePlotter.hh"
#include <BroadcastEvent.hh>
#include <IsoplotConfigDlg.hh>
#include <geometry2D.hh>

//
//.. constructor
//
MQIsolinePlotter::MQIsolinePlotter(ConstructArg arg, 
				   QWidget *parent, 
				   const char *name) :
  MQMeshPlotter(arg, parent, name)
{
  AddClassName();
  string event_name = "Need.";
  event_name += typeid(IsoValue).name();
  BroadcastEvent E(event_name);
  E.only_same_mesh = true;
  SendEvent(E);
  uint i;
  num_isos = E.NumOutPointers();
  ISO = new IsoValue* [num_isos];
  for (i = 0; i < num_isos; i++) {
    ISO[i] = dynamic_cast<IsoValue*>(E.OutPointer(i));
  };
  color = NULL;
  current_iso = 0;
  lines = true;
  fill = false;
  color_type = 2;
  colormap = false;
  //AutoLevels(10);
  num_levels = 10;
  level = new real[num_levels];
  for (i = 0; i < num_levels; i++) level[i] = 0;
  autolevels = true;
};

//
//.. SetLevels
//
void MQIsolinePlotter::SetLevels(real minv, real maxv, int nl)
{
  num_levels = nl;
  delete [] level;
  delete [] color;
  level = new real[num_levels];
  color = new QColor*[num_levels + 1];
  uint i;
  for (i = 0; i < num_levels; i++) {
    level[i] = minv + i * (maxv - minv) / (num_levels - 1);
  };
  switch(color_type) {
  case 0:
    for (i = 0; i < num_levels + 1; i++)
      color[i] = new QColor(255 - i * 180/(num_levels), 
			    255 - i * 180/(num_levels), 
			    255 - i * 180/(num_levels));
    break;
  case 1:
    for (i = 0; i < num_levels + 1; i++)
      color[i] = new QColor(55 + i * 180/(num_levels), 
			    0,
			    235 - i * 180/(num_levels));
    break;
  case 2:
    {
      real r, g, b, rv;
      for (i = 0; i < num_levels + 1; i++) {
	rv = ((real) i) / num_levels;
	r = 2*rv-1;
	if (rv < 0.5) g = 2*rv;
	else g = 2-2*rv;
	b = 1-2*rv;
	if (r > 1) r = 1;
	if (r < 0) r = 0;
	if (g > 1) g = 1;
	if (g < 0) g = 0;
	if (b > 1) b = 1;
	if (b < 0) b = 0;
// 	cout << i << " " << rv << " -> " << (int)(255*r) 
// 	     << " " << (int)(255*g) << " " << (int)(255*b) << endl;
	color[i] = new QColor((int)(255*r), (int)(255*g), (int)(255*b));
	color[i]->setHsv(240-(int)(rv*240), 150, 255);
      };
    };
    break;
  };
};

//
//.. AutoLevels
//
void MQIsolinePlotter::AutoLevels(int nl, MQPlotWidget *pw)
{
  real maxv=0, minv=0;
  bool first = true;
  real v, x, y, delta;
  FORALL(i, PL->) {
    x = PL->x(i);
    y = PL->y(i);
    if (pw->IsIn(x, y)) {
      v = ISO[current_iso]->Value(i);
      if (first) {
	first = false;
	maxv = v;
	minv = v;
      };
      if (v > maxv) maxv = v;
      if (v < minv) minv = v;
    };
  };
  delta = maxv - minv;
  delta /= (nl+1);
  maxv -= delta;
  minv += delta;
  SetLevels(minv, maxv, nl);
};

//
//.. CreateConfigDlg
//
QDialog* MQIsolinePlotter::CreateConfigDlg(QWidget *parent)
{ 
  return new IsoplotConfigDlg(this, parent);
};

//
//.. ComputePoint
//
void MQIsolinePlotter::ComputePoint(long int p0, long int p1, real v0, real v1, 
				    real iso, real &xiso, real &yiso)
{
  if (p0 > p1) {
    xiso = PL->x(p0) 
      + (iso - v0) / (v1 - v0) * (PL->x(p1) - PL->x(p0));
    yiso = PL->y(p0) 
      + (iso - v0) / (v1 - v0) * (PL->y(p1) - PL->y(p0));
  } else {
    xiso = PL->x(p1) 
      + (iso - v1) / (v0 - v1) * (PL->x(p0) - PL->x(p1));
    yiso = PL->y(p1) 
      + (iso - v1) / (v0 - v1) * (PL->y(p0) - PL->y(p1));
  }
  // simple test to check if the point is ok
  //if (DIST(PL->x(p0), PL->y(p0), xiso, yiso) > DIST(PL->x(p0), PL->y(p0), PL->x(p1), PL->y(p1))) {
  CheckReal(xiso);
  CheckReal(yiso);
};

//
//.. CreateLine
//
int MQIsolinePlotter::CreateLine(real iso, size_t *p, real *v, real *x, real *y)
{
  if (((v[0] >= iso) && (v[1] <= iso) && (v[2] <= iso)) 
      || ((v[0] <= iso) && (v[1] >= iso) && (v[2] >= iso))) {
    ComputePoint(p[0], p[1], v[0], v[1], iso, x[0], y[0]);
    ComputePoint(p[0], p[2], v[0], v[2], iso, x[1], y[1]);
    return 0;
  } else if (((v[1] >= iso) && (v[0] <= iso) && (v[2] <= iso))
	     || ((v[1] <= iso) && (v[0] >= iso) && (v[2] >= iso))) {
    ComputePoint(p[1], p[2], v[1], v[2], iso, x[0], y[0]);
    ComputePoint(p[1], p[0], v[1], v[0], iso, x[1], y[1]);
    return 1;
  } else if (((v[2] >= iso) && (v[0] <= iso) && (v[1] <= iso))
	     || ((v[2] <= iso) && (v[0] >= iso) && (v[1] >= iso))) {
    ComputePoint(p[2], p[0], v[2], v[0], iso, x[0], y[0]);
    ComputePoint(p[2], p[1], v[2], v[1], iso, x[1], y[1]);
    return 2;
  };
  return -1;
};

//
//.. DrawColorMap
//
void MQIsolinePlotter::DrawColorMap(MQPlotWidget *pw, QPainter *P, const QRect *R)
{
  uint i;
  int left_margin = 10;
  int top_margin = 10;
  int box_height = 25;
  int box_width = 90;
  int padding = 2;
  P->setPen(black);
  P->setBrush(white);
  P->drawRect(left_margin, top_margin, 
	      left_margin + box_width, (num_levels+1) * box_height);
  for (i = 0; i < num_levels + 1; i++) {
    P->setBrush(*color[i]);
    P->drawRect(left_margin , top_margin + i * box_height, 
		box_height, box_height);
  };
  QString s;
  for (i = 0; i < num_levels; i++) {
    s = s.setNum(level[i]);
    P->drawText(left_margin + box_height + padding,  
		top_margin + (i+1) * box_height + 2 * padding, s);
  };
};

//
//.. Plot
//
void MQIsolinePlotter::Plot(MQPlotWidget *pw, QPainter *P, const QRect *R)
{
  QPointArray p3(3);
  QPointArray p4(4);
  real x[2], y[2], v[3];
  size_t p[3];
  size_t l, n;
  size_t i0=0, i1=0, i2=0;
  int mode;
  int xi1, xi2, yi1, yi2;
  int xi[3], yi[3];
  CalcValues();
  P->setPen(black);
  bool draw;
  if (autolevels) AutoLevels(num_levels, pw);
  if (fill) {
    FORALL(i, EL->) {
      if (ElementInside(pw, i, R)) {
	for (n = 0; n < EL->NumNodes(i) - 2; n++) {
	  p[0] = EL->Node(i, 0);
	  p[1] = EL->Node(i, n+1);
	  p[2] = EL->Node(i, n+2);
	  v[0] = ISO[current_iso]->Value(p[0]);
	  v[1] = ISO[current_iso]->Value(p[1]);
	  v[2] = ISO[current_iso]->Value(p[2]);
	  pw->WorldToScreen(PL->x(p[0]), PL->y(p[0]), xi[0], yi[0]);
	  pw->WorldToScreen(PL->x(p[1]), PL->y(p[1]), xi[1], yi[1]);
	  pw->WorldToScreen(PL->x(p[2]), PL->y(p[2]), xi[2], yi[2]);
	  for (l = 0; l < num_levels + 1; l++) {
	    draw = false;
	    if (l == 0) draw = true;
	    else if (((v[0] >= level[l-1]) 
		      && (v[1] >= level[l-1]) 
		      && (v[2] >= level[l-1]))) draw = true;
	    if (!draw) break;
	  };
	  if (l > 0) l--;
	  P->setPen(*color[l]);
	  P->setBrush(*color[l]);
	  p3.setPoint(0, xi[0], yi[0]);
	  p3.setPoint(1, xi[1], yi[1]);
	  p3.setPoint(2, xi[2], yi[2]);
	  P->drawPolygon(p3);
	};
      };
    };
    for (l = 0; l < num_levels; l++) {
      FORALL(i, EL->) {
	if (ElementInside(pw, i, R)) {
	  for (n = 0; n < EL->NumNodes(i) - 2; n++) {
	    p[0] = EL->Node(i, 0);
	    p[1] = EL->Node(i, n+1);
	    p[2] = EL->Node(i, n+2);
	    v[0] = ISO[current_iso]->Value(p[0]);
	    v[1] = ISO[current_iso]->Value(p[1]);
	    v[2] = ISO[current_iso]->Value(p[2]);
	    pw->WorldToScreen(PL->x(p[0]), PL->y(p[0]), xi[0], yi[0]);
	    pw->WorldToScreen(PL->x(p[1]), PL->y(p[1]), xi[1], yi[1]);
	    pw->WorldToScreen(PL->x(p[2]), PL->y(p[2]), xi[2], yi[2]);

	    // <03 May 99 12:32:33 ogloth@alpine.inria.fr>
	    // an attempt to fix the problem with constant field plots
	    /*
	    bool constant_triangle = false;
	    real range = fabs(level[0] - level[num_levels-1]);
	    if (fabs(v[0]-v[1]) + fabs(v[0]-v[2]) + fabs(v[1]-v[2]) <= 1e-4*range) {
	      if (v[0] >= level[l]) break;
	      x[0] = PL->x(p[0]);
	      y[0] = PL->y(p[0]);
	      x[1] = PL->x(p[1]);
	      y[2] = PL->y(p[1]);
	      constant_triangle = true;
	      mode = 0;
	    } else {
	      // this was the original
	      mode = CreateLine(level[l], p, v, x, y);
	    };
	    */
	    try {
	      mode = CreateLine(level[l], p, v, x, y);
	      if (mode != -1) {
		pw->WorldToScreen(x[0], y[0], xi1, yi1);
		pw->WorldToScreen(x[1], y[1], xi2, yi2);
		switch(mode) {
		case 0:
		  i0 = 0;
		  i1 = 1;
		  i2 = 2;
		  break;
		case 1:
		  i0 = 1;
		  i1 = 2;
		  i2 = 0;
		  break;
		case 2:
		  i0 = 2;
		  i1 = 0;
		  i2 = 1;
		  break;
		};
		P->setPen(*color[l+1]);
		P->setBrush(*color[l+1]);
		//if ((v[i0] >= level[l]) || constant_triangle) {
		if (v[i0] >= level[l]) {
		  p3.setPoint(0, xi[i0], yi[i0]);
		  p3.setPoint(1, xi1, yi1);
		  p3.setPoint(2, xi2, yi2);
		  P->drawPolygon(p3);
		} else {
		  p4.setPoint(0, xi[i1], yi[i1]);
		  p4.setPoint(1, xi[i2], yi[i2]);
		  p4.setPoint(2, xi2, yi2);
		  p4.setPoint(3, xi1, yi1);
		  P->drawPolygon(p4);
		};
	      };
	    } catch (BadReal_error) {};
	  };
	};
      };
    };
  };
  if (lines) {
    FORALL(i, EL->) {
      if (ElementInside(pw, i, R)) {
	for (n = 0; n < EL->NumNodes(i) - 2; n++) {
	  p[0] = EL->Node(i, 0);
	  p[1] = EL->Node(i, n+1);
	  p[2] = EL->Node(i, n+2);
	  v[0] = ISO[current_iso]->Value(p[0]);
	  v[1] = ISO[current_iso]->Value(p[1]);
	  v[2] = ISO[current_iso]->Value(p[2]);
	  pw->WorldToScreen(PL->x(p[0]), PL->y(p[0]), xi[0], yi[0]);
	  pw->WorldToScreen(PL->x(p[1]), PL->y(p[1]), xi[1], yi[1]);
	  pw->WorldToScreen(PL->x(p[2]), PL->y(p[2]), xi[2], yi[2]);
	  P->setPen(black);
	  for (l = 0; l < num_levels; l++) {
	    try {
	      if (CreateLine(level[l], p, v, x, y) != -1) {
		pw->WorldToScreen(x[0], y[0], xi1, yi1);
		pw->WorldToScreen(x[1], y[1], xi2, yi2);
		P->drawLine(xi1, yi1, xi2, yi2);
	      };
	    } catch (BadReal_error) {};
	  };
	};
      };
    };
  };
  if (fill && colormap) DrawColorMap(pw, P, R);
};

void MQIsolinePlotter::CheckReal(real a)
{
  if (a == 0) return;
  a /= a;
  if (a == 1) return;
  throw BadReal_error();
};

#endif







