// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#include <mouse_config.h>
#ifdef WITH_QT

#include "MQMeshPlotter.hh"
#include <ConstructArg.hh>
#include <qpointarray.h>
#include <MQPlotWidget.hh>
#include <qpainter.h>

//
//.. constructor
//
MQMeshPlotter::MQMeshPlotter(ConstructArg arg, QWidget *parent, const char *name) : 
  MQPlotter(arg, parent, name)
{
  AddClassName();
  PL = FindObject<PointList>(false, true);
  EL = FindObject<ElementList>(false, true);
};

//
//.. MakeElement
//
QPointArray* MQMeshPlotter::MakeElement(MQPlotWidget *pw, long int e)
{
  QPointArray *pa = new QPointArray(EL->NumNodes(e) + 1);
  int x, y;
  unsigned int p;
  for (unsigned int n = 0; n < EL->NumNodes(e); n++) {
    p = EL->Node(e, n);
    pw->WorldToScreen(PL->x(p), PL->y(p), x, y);
    pa->setPoint(n, x, y);
  };
  pa->setPoint(EL->NumNodes(e), pa->point(0));
  return pa;
};

//
//.. ElementInside
//
bool MQMeshPlotter::ElementInside(MQPlotWidget *pw, long int e, 
				  const QRect *rect)
{
  size_t p;
  int xi1 = rect->left();
  int xi2 = rect->right();
  int yi1 = rect->bottom();
  int yi2 = rect->top();
  real x1, y1, x2, y2;
  pw->ScreenToWorld(xi1, yi1, x1, y1);
  pw->ScreenToWorld(xi2, yi2, x2, y2);
  for (unsigned int i = 0; i < EL->NumNodes(e); i++) {
    p = EL->Node(e, i);
    if ((PL->x(p) > x1) && (PL->x(p) < x2)
	&& (PL->y(p) > y1) && (PL->y(p) < y2)) {
      return true;
    };
  };
  return false;
};

//
//.. Plot
//
void MQMeshPlotter::Plot(MQPlotWidget *pw, QPainter *P, const QRect *R)
{
  P->setPen(blue);
  FORALL(i, EL->) {
    if (ElementInside(pw, i, R)) {
      QPointArray *pa = MakeElement(pw, i);
      P->drawPolyline(*pa);
      delete pa;
    };
  };
};

#endif








