// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#include <mouse_config.h>
#ifdef WITH_QT

#include "MQObjectTreeView.hh"
#include <MQObjectTreeItem.hh>

MQObjectTreeView::MQObjectTreeView(carg_t arg, QWidget* parent = NULL, const char* name = NULL) 
  : QListView(parent, name), MObject(arg)
{
  AddClassName();
  addColumn("class name");
  addColumn("object ID");
  addColumn("mesh ID");
  addColumn("garbage");
  setColumnWidthMode(0, Maximum);
  setColumnWidthMode(1, Maximum);
  setColumnWidthMode(2, Maximum);
  setColumnWidthMode(3, Maximum);
  setItemMargin(3);
};

void MQObjectTreeView::Add(QListViewItem *item, MObject *mobj)
{
  MObject *child = mobj->FirstChild();
  while (child != NULL) {
    QListViewItem* new_item = new MQObjectTreeItem(this, item, child);
    if (child != this) Add(new_item, child);
    child = child->NextSibling();
  };
};

void MQObjectTreeView::Update()
{
  clear();
  setSorting(0);
  QListViewItem* first_item = new MQObjectTreeItem(this, this, Ancestor());
  Add(first_item, Ancestor());
};

void MQObjectTreeView::HandleEvent(Event &E)
{
  MObject::HandleEvent(E);
  if (!E.Done()) {
    if (E.name == "ObjectTreeChanged") {
      E.React(this);
      QListViewItem *item = dynamic_cast<QListViewItem*>(E.sender);
      if (item == NULL) Update();
    };
  };
};

#endif









