// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#include <mouse_config.h>
#ifdef WITH_QT

#include "MQPlotWidget.hh"
#include <ConstructArg.hh>
#include <qpainter.h>
#include <qprinter.h>
#include <MQPlotter.hh>
#include <BroadcastEvent.hh>

MQPlotWidget::MQPlotWidget(ConstructArg arg, 
			   QWidget *parent, 
			   const char *name, 
			   WFlags f) :
  MQDrawWidget(arg, parent, name, f) 
{
  AddClassName();
  string event_name = "Need.";
  event_name += typeid(MQPlotter).name();
  BroadcastEvent E(event_name);
  E.only_same_mesh = true;
  SendEvent(E);
  int i;
  num_plotters = E.NumOutPointers();
  plotter_active = new bool [num_plotters];
  plotter = new MQPlotter* [num_plotters];
  for (i = 0; i < num_plotters; i++) {
    plotter[i] = dynamic_cast<MQPlotter*>(E.OutPointer(i));
    plotter_active[i] = false;
  };
};

MQPlotWidget::~MQPlotWidget()
{
  /*
  MQPlotter *p1, *p2;
  p1 = first_plotter;
  while (p1 != NULL) {
    p2 = p1->next;
    delete p1;
    p1 = p2;
  };
  */
};

void MQPlotWidget::paintEvent(QPaintEvent *E) 
{
  if (!is_painting) {
    is_painting = true;
    QPainter P(this);
    if (first_paint) {
      Autoscale();
      first_paint = false;
    };
    for (int i = 0; i < num_plotters; i++) {
      if (plotter_active[i]) plotter[i]->Plot(this, &P, &E->rect());
    };
    is_painting = false;
  };
};

void MQPlotWidget::Print()
{
  if (!is_painting) {
    is_painting = true;
    QPrinter qprint;
    if (first_paint) Autoscale();
    if (qprint.setup()) {
      string creator = PACKAGE;
      creator += " v";
      creator += VERSION;
      qprint.setCreator(creator.c_str());
      QPainter qpaint(&qprint);
      QRect R = qpaint.viewport();

      // scale the printout
      // use R for scaling and save old parameters
      int widget_w = rect().width();
      int widget_h = rect().height();
      int printer_w = qpaint.viewport().width();
      int printer_h = qpaint.viewport().height();
      float w_r = ((float)widget_h)/widget_w;
      float p_r = ((float)printer_h)/printer_w;
      real x_screen_orig = x_screen;
      real y_screen_orig = y_screen;
      other_scale = true;
      scale_rect = R;
      if (p_r > w_r) y_screen *= p_r/w_r;
      else x_screen *= w_r/p_r;

      // call all the plotters
      for (int i = 0; i < num_plotters; i++) {
	if (plotter_active[i]) plotter[i]->Plot(this, &qpaint, &R);
      };

      // reset the scaling
      other_scale = false;
      x_screen = x_screen_orig;
      y_screen = y_screen_orig;

      is_painting = false;
    };
  };
};

void MQPlotWidget::HandleEvent(Event &E)
{
  MQDrawWidget::HandleEvent(E);
  if (!E.Done()) {
    if (E.name == "RepaintEvent") {
      E.React(this);
      repaint();
    };
  };
};

#endif








