// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#include <mouse_config.h>
#ifdef WITH_QT

#include "MQVectorPlotter.hh"
#include <BroadcastEvent.hh>
#include <VectorplotConfigDlg.hh>

//
//.. constructor
//
MQVectorPlotter::MQVectorPlotter(ConstructArg arg, 
				 QWidget *parent, 
				 const char *name) :
  MQPlotter(arg, parent, name)
{
  AddClassName();
  PL = FindObject<PointList>(false, true);
  string ename = "Need.";
  ename += typeid(VectorValue).name();
  BroadcastEvent E(ename);
  E.only_same_mesh = true;
  SendEvent(E);
  int i;
  num_vecs = E.NumOutPointers();
  VEC = new VectorValue* [num_vecs];
  for (i = 0; i < num_vecs; i++) {
    VEC[i] = dynamic_cast<VectorValue*>(E.OutPointer(i));
    if (VEC[i] == NULL) {
      Error("got something which is not a 'VectorValue'");
    };
  };
  current_vec = 0;
  fixed = false;
  scaled = true;
  fixed_length = 5;
  scale_factor = 1.0;
};

//
//.. PointInside
//
bool MQVectorPlotter::PointInside(MQPlotWidget *pw, long int p, const QRect *rect)
{
  int xi1 = rect->left();
  int xi2 = rect->right();
  int yi1 = rect->bottom();
  int yi2 = rect->top();
  real x1, y1, x2, y2;
  pw->ScreenToWorld(xi1, yi1, x1, y1);
  pw->ScreenToWorld(xi2, yi2, x2, y2);
  real xp = PL->x(p);
  real yp = PL->y(p);
  return ((xp>=x1) && (xp<=x2) && (yp>=y1) && (yp<=y2));
};

//
//.. CreateConfigDlg
//
QDialog* MQVectorPlotter::CreateConfigDlg(QWidget *parent)
{
  return new VectorplotConfigDlg(this, parent);
};

//
//.. Plot
//
void MQVectorPlotter::Plot(MQPlotWidget *pw, QPainter *P, const QRect *R)
{
  real x0, y0, x1, y1, vx, vy, nx, ny, l;
  real ovx, ovy, vvx, vvy, x2, y2, x3, y3;
  int ix0, iy0, ix1, iy1, ix2, iy2, ix3, iy3; 
  CalcValues();
  FORALL(i, PL->) {
    x0 = PL->x(i);
    y0 = PL->y(i);
    if (pw->IsIn(x0, y0)) {
      vx = VEC[current_vec]->XValue(i);
      vy = VEC[current_vec]->YValue(i);
      // norm vector and check for zero
      l = pw->DiagonalLength() * sqrt(vx*vx + vy*vy);
      if (l > 1e-6) {
	nx = vx / l;
	ny = vy / l;
      } else {
	nx = 0;
	ny = 0;
      };
      if (scaled) {
	x1 = x0 + scale_factor * l * nx;
	y1 = y0 + scale_factor * l * ny;
      };
      if (fixed) {
	x1 = x0 + fixed_length * nx;
	y1 = y0 + fixed_length * ny;
      };
      vvx = x1 - x0;
      vvy = y1 - y0;
      ovx = .2*vvy;
      ovy = -.2*vvx;
      x2 = x1 - .3*vvx+ovx;
      x3 = x1 - .3*vvx-ovx;
      y2 = y1 - .3*vvy+ovy;
      y3 = y1 - .3*vvy-ovy;
      pw->WorldToScreen(x0, y0, ix0, iy0);
      pw->WorldToScreen(x1, y1, ix1, iy1);
      pw->WorldToScreen(x2, y2, ix2, iy2);
      pw->WorldToScreen(x3, y3, ix3, iy3);
      P->setPen(black);
      P->drawLine(ix0, iy0, ix1, iy1);
      P->drawLine(ix1, iy1, ix2, iy2);
      P->drawLine(ix1, iy1, ix3, iy3);
    };
  };
};

#endif




