// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#include "MeshSmoother2D.hh"
#include <geometry2D.hh>

//
//.. constructor
//
MeshSmoother2D::MeshSmoother2D(ConstructArg arg) : Mesh2DAction(arg)
{
  AddClassName();
  PL = FindObject<PointList>(false, true);
  EL = FindObject<ElementList>(false, true);
  PEL = FindObject<PelList2D>(false, true);
  new_x = new TList<real>(PL);
  new_y = new TList<real>(PL);
};

//
//.. CalcNewTriPoint
//
void MeshSmoother2D::CalcNewTriPoint(size_t p0, real &x_new, real &y_new) 
{
  real x0, y0;
  real x1, x2, xm, y1, y2, ym;
  real xn, yn;
  real nd_all;
  real pA, A;
  int n;
  size_t p1, p2;
  x_new = 0;
  y_new = 0;
  A = 0;
  nd_all = 0;
  // coordinates of the point p0, also used as origin for the transformation
  x0 = PL->x(p0);
  y0 = PL->y(p0);
  for (size_t i = 0; i < PEL->Size(p0); i++) {
    n = MESH->FindNode(PEL->At(p0, i), p0);
    p1 = EL->PermNode(PEL->At(p0, i), n + 1);
    p2 = EL->PermNode(PEL->At(p0, i), n - 1);
    //p1 = local_border->At (i);
    //p2 = local_border->At (i + 1);
    x1 = PL->x(p1);
    y1 = PL->y(p1);
    x2 = PL->x(p2);
    y2 = PL->y(p2);
//     Transform (p0, x1, y1);
//     Transform (p0, x2, y2);
    // normal vector of limiting edge
    xn = -(y2 - y1);
    yn = x2 - x1;
    // middle point of limiting edge
    xm = .5*(x1 + x2);
    ym = .5*(y1 + y2);
    // area of element with limiting edge
    pA = DIST (x1, y1, x2, y2) * .5 * (MESH->ND(p1) + MESH->ND(p2));
    //pA = DIST (x1, y1, x2, y2);
    //pA = 1.0;
    A += pA;
    x_new += pA * xm;
    y_new += pA * ym;
  };
  x_new /= A;
  y_new /= A;
//   Retransform (p0, x_new, y_new);
  //delete local_border;
};

//
//.. Act
//
void MeshSmoother2D::Act()
{
  unsigned int nt, nq;
  size_t j;
  real xn, yn;
  FORALL(i, PL->) {
    if ((PL->code(i) == 0) && !MESH->IsFixed(i) && MESH->IsMarked(i)) {
      nq = 0;
      nt = 0;
      for (j = 0; j < PEL->Size(i); j++) {
	if (EL->NumNodes(PEL->At(i, j)) == 4) nq++;
	if (EL->NumNodes(PEL->At(i, j)) == 3) nt++;
      };
      if (nq == 0) {
	CalcNewTriPoint(i, xn, yn);
	if (MESH->CheckPoint(i, xn, yn)) {
	  PL->x(i) = xn;
	  PL->y(i) = yn;
	};
      };
    };
  };
};














