// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#if !defined(MOUSEMAIN_HH)
#define MOUSEMAIN_HH

class MouseMain;

#include <MObjectBuilder.hh>
#include <MObject.hh>
#include <Action.hh>
#include <Communicator.hh>
#include <TList.hh>

extern Communicator TopObjectOfMouse;
extern size_t default_num_threads;

/**
   This is used to initialize a MOUSE application, reading in
   MCL code from a control file. 
*/
class MouseMain : public MObject
{
  friend class TextParser;

private:
  /** */ void StartStdMcl();
  /** */ TList<string> *file;
  /** */ bool parsing_file;
  /** */ size_t last_file;
  /** */ size_t last_line;

public:
  /** */ MouseMain();
  /** */ void main(Parser *P);
  /** */ void StartMouse(int argc, char* argv[]);
  /** */ static Communicator* Top() { return &TopObjectOfMouse; };
  /** */ static size_t NumThreads() { return default_num_threads; };
  /** */ static void SetNumThreads(size_t n) { default_num_threads = n; };
  /** */ bool IsParsingFile() { return parsing_file; };
  /** */ string LastFile() { return file->At(last_file); };
  /** */ size_t LastFileNumber() { return last_file; };
  /** */ size_t LastLine() { return last_line; };
};

#endif







