// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#ifndef MusclProjector2D2_HH
#define MusclProjector2D2_HH

class MusclProjector2D2;

#include <MusclProjector2D.hh>

/** */
class MusclProjector2D2 : public MusclProjector2D
{
protected:
  /** */ VarContainer::var2_t gx;
  /** */ VarContainer::var2_t gy;
  /** */ real edge_dx;
  /** */ real edge_dy;
  /** */ real scal_from;
  /** */ real scal_to;

public:
  /** */ MusclProjector2D2(ConstructArg arg);
  /** */ void operate(size_t ie, size_t ip, size_t from, size_t to, 
		      real nx, real ny, bool mod_from, bool mod_to);
};

inline void MusclProjector2D2::operate(size_t ie, size_t ip, 
				       size_t from, size_t to, 
				       real nx, real ny, 
				       bool mod_from, bool mod_to)
{
  edge_dx = (PL->x(to) - PL->x(from))*0.5;
  edge_dy = (PL->y(to) - PL->y(from))*0.5;
  for (iv = 0; iv < VL->NumVars(); iv++) {
    //.. compute the averages (central variables)
    var_h[iv][ie] = .5*( var_point[iv][from] + var_point[iv][to]);
    //.. scalar multiplication of variable gradients with edge vector
    scal_from = edge_dx * gx[iv][from] + edge_dy * gy[iv][from];
    scal_to = edge_dx * gx[iv][to] + edge_dy * gy[iv][to];
      // Extrapolation of plus variables (first order reduced by central)
    var_to[iv][ie] = (1. - upw) * var_h[iv][ie] + upw * (var_point[iv][to] - scal_to);
    var_from[iv][ie] = (1. - upw) * var_h[iv][ie] + upw * (var_point[iv][from] + scal_from);
  };
};

#endif







