// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#if !defined(PARAMETER_HH)
#define PARAMETER_HH

#include "MObject.hh"
#include <TKeyList.hh>
#include "VarList.hh"

/** 
    general base class to store problem parameters
*/
class Parameter : public MObject
{
protected:
  /** the real parameter values. */
  TKeyList<string, real> *real_param;

  /** flags. */
  TKeyList<string, bool> *flags;

  /** the integer parameter values. */
  TKeyList<string, int> *int_param;

  /** text missing. */
  void SendIntParam(Event &E, string name);

  /** text missing. */
  void SendRealParam(Event &E, string name);

  /** the free stream properties. */
  TList<real> *var_oo;

public:
  /** constructor. */
  Parameter(ConstructArg arg);

  /** get value for a real parameter. */
  real RealParam(string name);

  /** get value for a flag. */
  bool Flag(string name);

  /** get value for an integer parameter. */
  int IntParam(string name);

  /** text missing. */
  virtual void InitVars(VarList *VL);

  /** text missing. */
  real Var_oo(long int i) { return var_oo->At(i); };

  virtual void HandleEvent(Event &E);
};

#endif















