// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#include "Parser.hh"

Parser::Parser(MObject *a_parent) : MObject(a_parent)
{
  AddClassName();
};


//
//.. AddVar
//
void Parser::AddVar(string var_name, MObject *var_obj)
{ 
  try {
    VAR->Find(var_name);
    string text = "variable ";
    text += var_name;
    text += " has already been defined";
    Error(text);
  } catch (NotFound_error) {
    VAR->NewEntry(var_name) = var_obj;
    if (IsA<Parser>(parent)) {
      Parser *P = (Parser*) parent;
      P->AddVar(var_name, var_obj);
    };
  };
};

//
//.. Var
//
MObject* Parser::Var(string var_name)
{
  try {
    VAR->Find(var_name);
  }
  catch (NotFound_error) {
    Error("undefined variable '" + var_name + "'");
  };
  return VAR->At(var_name);
};

//
//.. ResolveVar
//
MObject* Parser::ResolveVar()
{
  string var_name = ReadString();
  if (var_name[0] != '$') Error("'$' expeceted");
  var_name[0] = '%';
  return Var(var_name);
};

//
//.. ReadBool
//
bool Parser::ReadBool()
{
  string word = ReadString();
  if (word == "true") { return true; };
  if (word == "false") { return false; };
  if (word == "yes") { return true; };
  if (word == "no") { return false; };
  Error("true/false or yes/no expected");
  return false; // avoid compiler warning
};







