// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#if !defined(PELLIST2D_HH)
#define PELLIST2D_HH

class PelList2D;

#include <PointList.hh>
#include <ElementList.hh>
#include <Flex2DArray.hh>

/** 
    This class provides point to element relation information. 
    It is purely twodimensional at the moment, so there might be some future
    changes to achive a better software structure. 
*/
class PelList2D : public MObject
{
protected:
  /** */ PointList *PL;
  /** */ ElementList *EL;

  /** the point to element information. */
  Flex2DArray<size_t> *PEL;

public:
  /** constructor. */
  PelList2D (ConstructArg arg);

  /** check if a point is in a certain element.
    @param point the index of the point
    @param element the index of the element */
  bool IsIn (size_t point, size_t element);

  /** create or update the information */
  void Create ();

  /** add a new element to a point.
    @param point the index of the point
    @param element the index of the element */
  void AddTo (size_t point, size_t element);

  /** remove all entries related to a certain element 
    @param element the index of the element */
  void DeleteFrom (size_t element);

  /** Create a TList<size_t> representing the outer surrounding of a point.
    @param point the index of the point */
  TList<size_t>* LocalBorder (size_t point);

  /** Create a TList<size_t> representing the inner surrounding of a point.
    @param point the index of the point */
  TList<size_t>* LocalInnerBorder (size_t point);

  /** Create a TList<size_t> representing a local pointo-to-other-point structure.
    @param point the index of the point */
  TList<size_t>* LocalPOP (size_t point);

  /** check if quadrangle are connected to a point
    @param point the index of the point 
    @return the number of quads connected */
  int QuadConnected (size_t point);

  /** check if triangles are connected to a point
    @param point the index of the point 
    @return the number of triangles connected */
  int TriConnected (size_t point);

  /** access to the At method of PEL. */
  size_t& At(size_t i, size_t j) { return PEL->At(i, j); };

  /** access to the Size method of PEL. */
  size_t Size(size_t i) { return PEL->Size(i); };
};

#endif








