// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#ifndef ReadPicFile_HH
#define ReadPicFile_HH

class ReadPicFile;

#include <Triangulator2D.hh>
#include <Mesh2DGeometryAction.hh>
#include <TList.hh>

/**
   This reads in a '.pic' file and tries to convert it into
   boundaries for mesh generation purposes.
*/
class ReadPicFile : public Mesh2DGeometryAction
{
protected:
  /** */
  struct Line {
    /** */ TList<real> *x;
    /** */ TList<real> *y;
    /** */ TList<int> *code;
    /** */ real min_dist;
    /** */ int std_code;
    /** */ Line(int a_code);
    /** */ void Add(real a_x, real a_y, int a_code=0);
    /** */ real FirstX() { return x->At(0); };
    /** */ real FirstY() { return y->At(0); };
    /** */ real LastX() { return x->At(x->NumEntries()-1); };
    /** */ real LastY() { return y->At(y->NumEntries()-1); };
    /** */ real X(size_t i) { return (*x)[i]; };
    /** */ real Y(size_t i) { return (*y)[i]; };
    /** */ void Merge(Line *line, int new_code);
    /** */ void ComputeMin();
  };

  /** */ string filename;
  /** */ void Process(string text_line, Line* &line);
  /** */ int new_code;

public:
  /** */ ReadPicFile(carg_t arg);
  /** */ virtual void Act();
};

#endif

