// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#include <TextParser.hh>
#include <IStreamParser.hh>
#include <EmptyParser.hh>
#include <stdio.h>
#include <MouseMain.hh>

TextParser::TextParser (MObject *a_parent) : Parser(a_parent)
{
  AddClassName();
  buffer = "";
  i_stream = NULL;
  current_word = 0;
  VAR = new TKeyList<string, MObject*>(10, 10, "", NULL);
  mmain = FindObject<MouseMain>();
  MacroName = new TList<string>(20, 10, "");
  MacroText = new TList<string>(MacroName, "");
  MacroParam = new TList<TList<string>*>(MacroName, NULL);
  IsRMacro = new TList<bool>(MacroName, false);
  RCounter = new TList<int>(MacroName, 0);
};

void TextParser::InsertTag(size_t i_line, size_t i_file, string &write_buffer)
{
  char word[100];
  strcpy(word, "");
  sprintf(word, " %d %d ", i_line, i_file);
  write_buffer += word;
};

void TextParser::AppendWord(string app_word, string &write_buffer)
{
  InsertTag(mmain->LastLine(), mmain->LastFileNumber(), write_buffer);
  write_buffer += app_word;
};

void TextParser::ReadFile(string filename, string &read_buffer)
{
  char c;
  bool comment_flag = false;
  bool pre_comment_flag = false;
  bool was_space = true;
  ifstream *s;
  s = new ifstream(filename.c_str());
  if (s->fail()) {
    delete s;
    string mouse_home;
    if (getenv("MOUSEHOME") == NULL) {
      Error("environment variable MOUSEHOME not found");
    } else {
      mouse_home = string(getenv("MOUSEHOME"));
    };
    string complete_filename = mouse_home + "/include/" + filename;
    s = new ifstream(complete_filename.c_str());
    if (s->fail()) {
      Error("couldn't open '" + filename + "'");
    };
  };
  size_t i_file = mmain->file->AddEntry();
  mmain->file->At(i_file) = filename;
  size_t i_line = 1;
  while (!s->fail()) {
    s->get(c);
    if (!comment_flag) {
      if (!pre_comment_flag) {
	if (c == '/') pre_comment_flag = true;
      } else {
	if (c == '/') comment_flag = true;
	else {
	  pre_comment_flag = false;
	  if (was_space) InsertTag(i_line, i_file, read_buffer);
	  read_buffer += '/';
	  was_space = false;
	};
      };
      if (!pre_comment_flag && !comment_flag) {
	if ((c == '{') || (c == '}')) {
	  read_buffer = read_buffer + ' ';
	  was_space = true;
	};
	if (was_space && !isspace(c)) InsertTag(i_line, i_file, read_buffer);
	was_space = isspace(c);
	read_buffer = read_buffer + c;
	if ((c == '{') || (c == '}')) {
	  read_buffer = read_buffer + ' ';
	  was_space = true;
	};
      };
    };
    if (c == '\n') {
      comment_flag = false;
      pre_comment_flag = false;
      i_line++;
      was_space = true;
    };
  };
  delete s;
};

void TextParser::Include()
{
  string word;
  string new_buffer = "";
  Reset();
  while (!eof()) {
    word = ReadString();
    bool processed = false;
    if ((word == "#include") || (word == "#package")) {
      string filename = ReadString();
//       if (word == "#package") {
// 	filename = string(getenv("MOUSEDIR")) + "/packages/" + filename + ".pack";
//       };
      cout << "including '" + filename + "'" << endl;
      ReadFile(filename, new_buffer);
      processed = true;
    }
    if (!processed) AppendWord(word, new_buffer);
  }
  buffer = new_buffer;
  Reset();
};

void TextParser::FindMacros()
{
  Reset();
  string new_buffer = "";
  string word;
  {
    while (!eof()) {
      word = ReadString();
      bool macro_found = ((word == "#macro") || (word == "#rmacro"));
      //if (word == "#macro") {
      if (macro_found) {
	size_t i = MacroName->AddEntry();
	if (word == "#rmacro") {
	  IsRMacro->At(i) = true;
	  RCounter->At(i) = 0;//??
	} else {
	  IsRMacro->At(i) = false;
	};
	word = ReadString();
	MacroName->At(i) = word;
	MacroText->At(i) = "";
	MacroParam->At(i) = new TList<string>(5, 5, "");
	{
	  Parser *SP = SubParser();
	  while (!SP->eof()) {
	    word = SP->ReadString();
	    size_t ii = MacroParam->At(i)->AddEntry();
	    MacroParam->At(i)->At(ii) = word;
	  }
	}
	{
	  Parser *SP = SubParser();
	  while (!SP->eof()) {
	    word = SP->ReadString();
	    AppendWord(word, MacroText->At(i));
	  }
	}
      } else {
	AppendWord(word, new_buffer);
      }
    }
  }
  buffer = new_buffer;
  Reset();
};

void TextParser::EvalCond()
{
  Reset();
  bool changed = true;
  string new_buffer, word;
  while (changed) {
    new_buffer = "";
    changed = false;
    while (!eof()) {
      word = ReadString();
      if (word == "#ifequal") {
	string expr1 = ReadString();
	string expr2 = ReadString();
	Parser *SP1 = SubParser();
	Parser *SP2 = SubParser();
	Parser *SP;
	if (expr1 == expr2) SP = SP1;
	else SP = SP2;
	while (!SP->eof()) {
	  word = SP->ReadString();
	  AppendWord(word, new_buffer);
	};
	delete SP;
      } else {
	AppendWord(word, new_buffer);
      };
    };
    buffer = new_buffer;
    Reset();
  };
};

void TextParser::ExpandMacros()
{
  Reset();
  bool changed = true;
  string new_buffer, word;
  uint expand_number = 0;
  while (changed) {
    new_buffer = "";
    changed = false;
    while (!eof()) {
      word = ReadString();
      bool macro_expanded = false;

      // check if word is the name of a macro
      FORALL(i, MacroName->) {

	// if so expand it in the text
	if (MacroName->At(i) == word) {
	  expand_number++;
	  string macro_name = word;

	  // read in the counter for recursive macros
	  int rcount = 1;
	  if (IsRMacro->At(i)) {
	    rcount = ReadInt();
	    if (rcount > 0) RCounter->At(i) = rcount;
	    else rcount *= -1;
	  };

	  // read the parameters into a list
	  TList<string> ParamValue(5, 5, "");
	  {
	    int num_brackets = 0;
	    size_t ie;
	    bool start = true;
	    Parser *SP = SubParser();
	    while (!SP->eof()) {
	      word = SP->ReadString();
	      if (word == "{") { 
		if (start) {
		  Error("syntax error");
		} else {
		  start = false;
		  AppendWord(" { ", ParamValue[ie]);
		  num_brackets++;
		};
	      } else if (word == "}") {
		if (start) {
		  Error("syntax error");
		} else {
		  start = false;
		  AppendWord(" } ", ParamValue[ie]);
		  num_brackets--;
		};
	      } else if (num_brackets == 0) {
		ie = ParamValue.AddEntry();
		start = false;
		ParamValue[ie] = "";
		AppendWord(word, ParamValue[ie]);
		if ((word[0] == '%') || (word[0] == '^')) {
		  word = SP->ReadString();
		  if ((word == "{") || (word == "}")) Error("syntax error");
		  AppendWord(word, ParamValue[ie]);
		};
	      } else {
		if (start) {
		  Error("syntax error");
		} else {
		  start = false;
		  AppendWord(word, ParamValue[ie]);
		};
	      }
	    };
	    if (ParamValue.NumEntries() != MacroParam->At(i)->NumEntries()) {
	      Error("parameter list for macro '" + macro_name + "' does not match");
	    }
	    if (num_brackets != 0) Error("unmatched parenthesis");
	  }

	  // insert the macro in the new buffer
	  if (rcount > 0) {
	    string macro_text = MacroText->At(i);
	    IStreamParser P(this);
	    P.Add(macro_text);
	    bool param_found;
	    while (!P.eof()) {
	      word = P.ReadString();
	      // check if a macro calls itself
	      if (word == macro_name) {
		if (IsRMacro->At(i)) {
		  char rcount_text[20];
		  sprintf(rcount_text, "%d", -(rcount - 1));
		  string rcount_string = rcount_text;
		  AppendWord(macro_name, new_buffer);
		  AppendWord(rcount_string, new_buffer);
		} else {
		  Error("a macro tries to call itself (use '#rmacro' for recursive structures)");
		}
	      } 
	      // check for the '#rcount' statement
	      // #rcount is used to get the current recursion level
	      else if (word == "#rcount") {
		char rcount_text[20];
		sprintf(rcount_text, "%d", RCounter->At(i) - rcount + 1);
		string rcount_string = rcount_text;
		AppendWord(rcount_string, new_buffer);
	      } 
	      // check for the '#maxrcount' statement
	      // #maxrcount is used to get maximal recursion level
	      else if (word == "#maxrcount") {
		char maxrcount_text[20];
		sprintf(maxrcount_text, "%d", RCounter->At(i));
		string maxrcount_string = maxrcount_text;
		AppendWord(maxrcount_string, new_buffer);
	      } else {
		// check if word is a parameter from the list
		param_found = false;
		FORALL(ii, MacroParam->At(i)->) {
		  if (MacroParam->At(i)->At(ii) == word) {
		    new_buffer = new_buffer + " " + ParamValue[ii];
		    param_found = true;
		    break;
		  }
		  if (MacroParam->At(i)->At(ii) == DelFirstChar(word)) {
		    istrstream s(ParamValue[ii].c_str());
		    int count = 0;
		    while (!s.eof()) {
		      string pv_word;
		      s >> pv_word;
		      count++;
		      if (count == 3) new_buffer = new_buffer + " " + word[0] + pv_word;
		      else new_buffer = new_buffer + " " + pv_word;
		    };
		    //new_buffer = new_buffer + " " + word[0] + ParamValue[ii];
		    param_found = true;
		    break;
		  }
		}
		// for not parameter words
		if (!param_found) {
		  if (word[0] == '^') {
		    word[0] = '%';
		    char append[100];
		    strcpy(append, "");
		    sprintf(append, "_macro%d", expand_number);
		    word += append;
		  };
		  if (word[0] == '&') {
		    word[0] = '$';
		    char append[100];
		    strcpy(append, "");
		    sprintf(append, "_macro%d", expand_number);
		    word += append;
		  };
		  AppendWord(word, new_buffer);
		};
	      }
	    }
	  } else {
	    // add a NopAction if this is the end of a recursive macro tree
	    AppendWord("NopAction", new_buffer);
	    AppendWord("{", new_buffer);
	    AppendWord("}", new_buffer);
	  };
	  changed = true;
	  macro_expanded = true;
	  break;
	}
      }
      if (!macro_expanded) AppendWord(word, new_buffer);
    }
    buffer = new_buffer;
    Reset();
    Concat();
    EvalCond();
    //Out(cout);
  }
};

/*
void TextParser::PreProcess ()
{
  uint expand_number = 0;
  size_t i, ii;
  string new_buffer = "";
  string word;
  TList<string> MacroName(20, 10, "");
  TList<string> MacroText(&MacroName, "");
  TList<TList<string>*> MacroParam(&MacroName, NULL);

  Include();

  // find macros
  Reset();
  new_buffer = "";
  {
    while (!eof()) {
      word = ReadString();
      if (word == "#macro") {
	word = ReadString();
	i = MacroName.AddEntry();
	MacroName.At(i) = word;
	MacroText.At(i) = "";
	MacroParam.At(i) = new TList<string>(5, 5, "");
	{
	  Parser *SP = SubParser();
	  while (!SP->eof()) {
	    word = SP->ReadString();
	    ii = MacroParam.At(i)->AddEntry();
	    MacroParam.At(i)->At(ii) = word;
	  }
	}
	{
	  Parser *SP = SubParser();
	  while (!SP->eof()) {
	    word = SP->ReadString();
	    //MacroText.At(i) = MacroText.At(i) + word + " ";
	    AppendWord(word, MacroText.At(i));
	  }
	}
      } else {
	//new_buffer = new_buffer + " " + word;
	AppendWord(word, new_buffer);
      }
    }
  }
  buffer = new_buffer;
  Reset();
  // expand macros
  bool changed = true;
  while (changed) {
    new_buffer = "";
    changed = false;
    while (!eof()) {
      word = ReadString();
      bool macro_expanded = false;
      for(size_t i=MacroName.begin(); i<MacroName.end(); i=MacroName.next(i)) {
	if (MacroName.At(i) == word) {
	  expand_number++;
	  string macro_name = word;
	  TList<string> ParamValue(5, 5, "");
	  {
	    int num_brackets = 0;
	    size_t ie;
	    bool start = true;
	    Parser *SP = SubParser();
	    while (!SP->eof()) {
	      word = SP->ReadString();
	      if (word == "{") { 
		if (start) {
		  Error("syntax error");
		} else {
		  start = false;
		  //ParamValue.At(ie) = ParamValue.At(ie) + " { ";
		  AppendWord(" { ", ParamValue[ie]);
		  num_brackets++;
		};
	      } else if (word == "}") {
		if (start) {
		  Error("syntax error");
		} else {
		  start = false;
		  //ParamValue.At(ie) = ParamValue.At(ie) + " } ";
		  AppendWord(" } ", ParamValue[ie]);
		  num_brackets--;
		};
	      } else if (num_brackets == 0) {
		ie = ParamValue.AddEntry();
		start = false;
		//ParamValue.At(ie) = word;
		ParamValue[ie] = "";
		AppendWord(word, ParamValue[ie]);
		if ((word[0] == '%') || (word[0] == '^')) {
		  word = SP->ReadString();
		  if ((word == "{") || (word == "}")) Error("syntax error");
		  //ParamValue.At(ie) = ParamValue.At(ie) + " " + word;
		  AppendWord(word, ParamValue[ie]);
		};
	      } else {
		if (start) {
		  Error("syntax error");
		} else {
		  start = false;
		  //ParamValue.At(ie) = ParamValue.At(ie) + " " + word;
		  AppendWord(word, ParamValue[ie]);
		};
	      }
	    };
	    if (ParamValue.NumEntries() != MacroParam.At(i)->NumEntries()) {
	      Error("parameter list for macro '" + macro_name + "' does not match");
	    }
	    if (num_brackets != 0) Error("unmatched parenthesis");
	  }
	  {
	    string macro_text = MacroText.At(i);
	    //istrstream s(macro_text.c_str());
	    IStreamParser P(this);
	    P.Add(macro_text);
	    bool param_found;
	    while (!P.eof()) {
	      //s >> word;
	      word = P.ReadString();
	      //if (s.good()) {
	      {
		param_found = false;
		  FORALL(ii, MacroParam.At(i)->) {
		  if (MacroParam.At(i)->At(ii) == word) {
		    new_buffer = new_buffer + " " + ParamValue.At(ii);
		    param_found = true;
		    break;
		  }
		  if (MacroParam.At(i)->At(ii) == DelFirstChar(word)) {
		    new_buffer = new_buffer + " " + word[0] + ParamValue.At(ii);
		    param_found = true;
		    break;
		  }
		}
		if (!param_found) {
		  if (word[0] == '^') {
		    word[0] = '%';
		    char append[100];
		    strcpy(append, "");
		    sprintf(append, "_macro%d", expand_number);
		    word += append;
		  };
		  if (word[0] == '&') {
		    word[0] = '$';
		    char append[100];
		    strcpy(append, "");
		    sprintf(append, "_macro%d", expand_number);
		    word += append;
		  };
		  //new_buffer = new_buffer + " " + word;
		  AppendWord(word, new_buffer);
		};
	      }
	    }
	  }
	  changed = true;
	  macro_expanded = true;
	  break;
	}
      }
      //new_buffer = new_buffer + " " + word;
      if (!macro_expanded) AppendWord(word, new_buffer);
    }
    buffer = new_buffer;
    Reset();
  }
  for(size_t i=MacroParam.begin(); i<MacroParam.end(); i=MacroParam.next(i)) {
    delete MacroParam.At(i);
  };
  Concat();
};
*/

void TextParser::PreProcess()
{
  Include();
  FindMacros();
  Concat();
  EvalCond();
  ExpandMacros();
  //Out(cout);
};

void TextParser::Concat()
{
  string new_buffer = "";
  Reset();
  while (!eof()) {
    string word = ReadString();
    if (word == "#concat") {
      Parser *SP = SubParser();
      string concat_word = "";
      while (!SP->eof()) concat_word += SP->ReadString();
      AppendWord(concat_word, new_buffer);
    } else {
      AppendWord(word, new_buffer);
    };
  };
  buffer = new_buffer;
  Reset();
};

void TextParser::eatwhite()
{
  char c;
  while (i_stream->get(c)) {
    if (isspace(c) == 0) {
      i_stream->putback(c);
      break;
    };
  };
};

void TextParser::Reset ()
{
  delete i_stream;
  i_stream = new istrstream(buffer.c_str());
};

template<class T>
T TextParser::Read()
{
  eatwhite();
  T value;
  *i_stream >> mmain->last_line >> mmain->last_file >> value;
  if (i_stream->fail()) Error("syntax error");
  eatwhite();
  return value;
};

Parser* TextParser::SubParser ()
{
  if (eof()) return new EmptyParser(this);
  string sub_buffer, word;
  sub_buffer = "";
  word = ReadString();
  if (word != "{") Error("'{' expected");
  word = ReadString();
  int num_words = 0;
  int num_brackets = 0;
  while (((word != "}") || (num_brackets > 0)) && !S().fail()) {
    if (word == "{") {
      num_brackets++;
    } else if (word == "}") {
      num_brackets--;
    };
    AppendWord(word, sub_buffer);
    num_words++;
    if (!S().fail()) {
      word = ReadString();
    };
  };
  if (word != "}") Error("unexpected end of text");

//   if (num_words == 0) return NULL;

  IStreamParser *P = new IStreamParser(this);
  P->Add(sub_buffer);

  //size_t i;
  string var_name;
  //_FORALL(i, VAR->) {
  for(size_t i=VAR->begin(); i<VAR->end(); i=VAR->next(i)) {
    var_name = VAR->operator[](i);
    P->VAR->NewEntry(var_name) = VAR->At(var_name);
  };
    
  P->DefineAsGarbage();
  return P;
};

void TextParser::Out(ostream &os)
{
  int i = 0;
  int ii;
  istrstream s(buffer.c_str());
  string word;
  while (!s.eof()) {
    s >> word;
    if (word == "}") i -= 2;
    for (ii = 0; ii < i; ii++) os << " ";
    os << word << endl;
    if (word == "{") i += 2;
  };
};

string TextParser::DelFirstChar(string word)
{
  string new_word = "";
  for(size_t i=1; i < word.size(); i++) new_word += word[i];
  return new_word;
};














