// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#if !defined(TextParser_HH)
#define TextParser_HH

#include <iostream.h>
#include <fstream.h>
#include <strstream.h>

class TextParser;

#include <TKeyList.hh>
#include <Parser.hh>
#include <ConstructArg.hh>
#include <MouseMain.hh>

/**
   base class for control code parsing. \\
   This class should not be instantiated using the control file at the moment. 
*/
class TextParser : public Parser
{
protected:
  /** */ string buffer;
  /** */ istrstream *i_stream;
  /** */ int current_word;
  /** */ void InsertTag(size_t i_line, size_t i_file, string &read_buffer);
  /** */ void ReadFile(string filename, string &read_buffer);
  /** */ void Include();
  /** */ void eatwhite();
  /** */ string DelFirstChar(string word);
  /** */ MouseMain *mmain;
  /** */ void AppendWord(string word, string &write_buffer);

  /** */ TList<string> *MacroName;
  /** */ TList<string> *MacroText;
  /** */ TList<TList<string>*> *MacroParam;
  /** */ TList<bool> *IsRMacro;
  /** */ TList<int> *RCounter;

public:
  /** */ TextParser(MObject *a_parent);
  /** */ virtual ~TextParser() { 
    delete i_stream;
  };

  /** find macro definitions */
  void FindMacros();

  /** perform macro expansions */
  void ExpandMacros();

  /** perform macro extension and remove comments */
  void PreProcess ();

  /** perform string concatenations */
  void Concat ();

  /** evaluate # conditionals */
  void EvalCond ();

  /** */ void Reset ();
  /** */ istrstream& S() { return *i_stream; };

  /** */ template<class T> T Read();
  /** */ template<class T> void Read(T &value) { value = Read<T>(); };
  /** */ virtual int ReadInt() { return Read<int>(); };
  /** */ virtual size_t ReadIndex(){ return Read<size_t>(); };
  /** */ virtual string ReadString(){ return Read<string>(); };
  /** */ virtual real ReadReal(){ return Read<real>(); };
  /** */ virtual Parser* SubParser ();

  /** @return the eof flag of the input stream of the Parser. */
  virtual bool eof() { 
    eatwhite();
    return i_stream->eof(); 
  };

  /** @return the fail flag of the input stream of the Parser. */
  int fail() { return i_stream->fail(); };

  /** @return the good flag of the input stream of the Parser. */
  int good() { return i_stream->good(); };

  /** output utility for debugging. */
  void Out(ostream &s);
};



#endif











