// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik
// Universitaet Duisburg, Germany
// Institute for Combustion and Gas Dynamics
// University of Duisburg, Germany
// Tue Jul 13 1999
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!
#ifndef TurkelPrec2D_HH
#define TurkelPrec2D_HH

#include <mouse_config.h>

class TurkelPrec2D;

#include <mouse_types.hh>
#include <ConstructArg.hh>
#include <CompressibleOperator2D.hh>

/**
   A preconditioner for low mach number flow.
   Thanks to Ceclile Viozat for the implementation in MOUSE.
*/
class TurkelPrec2D : public CompressibleOperator2D
{
protected:

public:
  /** */ TurkelPrec2D (carg_t arg);
  /** */ virtual ~TurkelPrec2D ();
  /** */ void operate (size_t i_tmp, size_t i);
};

inline void TurkelPrec2D::operate(size_t i_tmp, size_t i)
{
  //
  //.... Transform cons. residuals according to Report INRIA 3084 pp15
  //     (4x4 matrix right here)
  //...... Some useful quantities
  real u = rhou[i] / rho[i];
  real v = rhov[i] / rho[i];
  real v_norm2 = u*u + v*v;
  real press = gamma1 * (rhoE[i] - 0.5 * v_norm2 * rho[i]);
  real c_speed2 = press * gamma / rho[i];
  real enthalpie = (rhoE[i] + press) / rho[i];
  real beta2 = Mach_oo*Mach_oo;
  if(beta2 < 0.000001) {
    beta2 = 0.000001;
  };
  //...... helping quantities
  real help_A = (beta2-1) * gamma1 / c_speed2;
  real help_B = v_norm2 * 0.5 * res_rho[i] 
    - u * res_rhou[i] - v * res_rhov[i] + res_rhoE[i];
  //...... transformed residuals
  res_rho[i] += help_A * help_B;
  res_rhou[i] += help_A * help_B * u;
  res_rhov[i] += help_A * help_B * v;
  res_rhoE[i] += help_A * help_B * enthalpie;
};

#endif










