// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#include "WriteTecplotFile2D.hh"
#include <CppConstructArg.hh>

WriteTecplotFile2D::WriteTecplotFile2D(ConstructArg arg) : Action(arg)
{
  AddClassName();
  PL = FindObject<PointList>(false, true);
  VL = FindObject<VarList>(false, true);
  tec_filename = arg.P->ReadString();
  el_filename = arg.P->ReadString();
  fieldname = arg.P->ReadString();
};

void WriteTecplotFile2D::Act()
{
  VarContainer::var2_t var = VL->Get2DVar(fieldname);
  bool EL_found = true;
  ElementList *EL;
  EL = LookForObject<ElementList>(false, true);
  if (EL == NULL) {
    EL_found = false;
    EL = new ElementList(ccarg_t(this) 
			 + size_t(PL->MaxNumEntries())
			 + size_t(PL->DeltaEntries())
			 + size_t(2)
			 + el_filename
			 + true);
  };
  ofstream f((tec_filename + ".tec").c_str());
  f << "TITLE = \"" << tec_filename << "\"" << endl;
  f << "VARIABLES = \"X\", \"Y\"";
  for (size_t j = 0; j < VL->NumVars(); j++) {
    f << ", \"" << VL->VarName(j) << "\"";
  };
  f << endl;
  f << "ZONE N=" << VL->NumEntries() << ", E=" << EL->NumEntries()
    << ", F=FEPOINT, ET=QUADRILATERAL" << endl;
  FORALL(i, PL->) {
    f << PL->x(i) << " " << PL->y(i);
    for (size_t j = 0; j < VL->NumVars(); j++) {
      f << " " << var[j][i];
    };
    f << endl;
  };
  FORALL(i, EL->) {
    f << EL->Node(i,0)+1 << " "
      << EL->Node(i,1)+1 << " "
      << EL->Node(i,2)+1 << " ";
    if (EL->NumNodes(i) == 3) f << EL->Node(i,2)+1;
    if (EL->NumNodes(i) == 4) f << EL->Node(i,3)+1;
    f << endl;
  };
  f << endl;
  system(("preplot " + tec_filename + ".tec").c_str());
  Mesg("tecplot file written.");
  if (!EL_found) delete EL;
};







