// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#if !defined(GEOMETRY2D)
#define GEOMETRY2D

#include <math.h>
#include <mouse_types.hh>

int dummy_geometry2D();

/**(geometry2D)\\
 text missing.\\
 :
 */
#define SQR(X) ((X) * (X))

/**(geometry2D)\\
  compute the distance between two points.\\
  :
  @param X1 x component of the first point
  @param Y1 y component of the first point
  @param X2 x component of the second point
  @param Y2 y component of the second point
  @return the distance between the two points
 */
#define DIST(X1,Y1,X2,Y2) (sqrt( SQR(((X2) - (X1))) + SQR(((Y2) - (Y1)))))

/**
   compute the absolute value of a 2D vector.
   :
   @param X the x component of the vector
   @param Y the y component of the vector
   @return the absoulte value
*/
#define VABS2(X,Y) (sqrt((X)*(X)+(Y)*(Y)))

/**
   normalize a 2D vector.
   :
   @param X the x component of the vector
   @param Y the y component of the vector
*/
#define VNORM2(X,Y) {				\
  real l = VABS2((X),(Y));			\
  (X) /= l;					\
  (Y) /= l;					\
};


/**
   :
*/
#define COSANGLE(X1,Y1,X2,Y2) (((X1)*(X2)+(Y1)*(Y2))/\
(sqrt((X1)*(X1)+(Y1)*(Y1))*sqrt((X2)*(X2)+(Y2)*(Y2)))) 


/**
   compute geometric angles.
   :
   @param ax x component of the first vector
   @param ay y component of the first vector
   @param bx x component of the second vector
   @param by y component of the second vector
   @return the counter clockwise angle between the vectors [ax,ay] and [bx,by]
*/
double Angle (double ax, double ay, double bx, double by);

/**(geometry2D)\\
  compute the intersection between two straight lines.
  :
  @param k parameter for the first line
  @param l parameter for the second line
  @param rx1 x component of the first origin vector
  @param ry1 y component of the first origin vector
  @param ux1 x component of the first dircetional vector
  @param uy1 y component of the first directional vector
  @param rx2 x component of the second origin vector
  @param ry2 y component of the second origin vector
  @param ux2 x component of the second directional vector
  @param uy2 y component of the second directional vector
  */
int Intersection (double& k, double &l,
		  double rx1, double ry1,
		  double ux1, double uy1,
		  double rx2, double ry2,
		  double ux2, double uy2);

/** turn a 2D vector clockwise by 90deg. 
    @param x the x component of the vector
    @param y the x component of the vector */
inline void TurnLeft(real &x, real &y)
{
  real hx = x;
  x = -y;
  y = hx;
};

/** turn a 2D vector counter clockwise by 90deg. 
    @param x the x component of the vector
    @param y the x component of the vector */
inline void TurnRight(real &x, real &y)
{
  real hx = x;
  x = y;
  y = -hx;
};

/**
   convert polar coordiantes to cartesian coordinates.
   @param r the radius
   @param alpha the angle
   @param x0 the x origin in cartesian coordinates
   @param y0 the y origin in cartesian coordinates
   @param x the cartesian x component
   @param y the cartesian y component 
*/
inline void PolToCart(real r, real alpha, real x0, real y0,
		      real &x, real &y)
{
  x = x0 + r*cos(alpha);
  y = y0 + r*sin(alpha);
};

void CircumCircle (double&, double&, double&,
		   double, double, double, double, double, double);

void ComputeNadir (double &x1, double &y1,
		   double &x2, double &y2,
		   double &xp, double &yp,
		   double &xn, double &yn, double ls);



#endif







