// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#include <FindMinMax.hh>
#include <CellProjection2D.hh>

template<class POP>
AggloMesh2DSlimBound<POP>::AggloMesh2DSlimBound(ConstructArg arg) : AggloMesh2DBasic(arg)
{
  AddClassName();
};

template<class POP>
void AggloMesh2DSlimBound<POP>::BoundaryFront(size_t P1)
{
  if (!marked[P1]) {
    // find the first free neighbour
    size_t nb_Bneigh = POPL_F->NumBLinks(P1);
    for (size_t nghb = 0; nghb < nb_Bneigh; nghb++) {
      size_t P2 = POPL_F->BLinkPoint(P1, nghb);
      if (!marked[P2]) {
	// the agglomerate is at lest composed of these two nodes
	marked[neighbour] = true;
	
	break
      }
    }
  }
};

template<class POP>
void AggloMesh2DSlimBound<POP>::Agglomerate()
{
  TList<bool> marked(PL_F,false);

  // First we agglomerate only boundary nodes together
  size_t done = 0; // the number of nodes agglomerated in current level
  size_t cur_b = BPL_F->begin();

  while (done < BPL_F->NumEntries()) {
    size_t cur_node = BPL_F->FIndex(cur_b); // field index
    // a node to agglomerate ?
    if (!marked[cur_node]) {
      marked[cur_node] = true;

      // create a new coarse node
      size_t cur_agglo = PL->AddEntry();
      PL->SetIVol(cur_agglo, PL_F->Vol(cur_node));

      // this is bogus - will be changed !!!
      PL->x(cur_agglo) = PL_F->x(cur_node);
      PL->y(cur_agglo) = PL_F->y(cur_node);
      PL->code(cur_agglo) = PL_F->code(cur_node);

      
      // agglomerate all free neighbours
      size_t nb_neighbour = POPL_F->NumBLinks(cur_node);
      for (size_t cur_nghb = 0; cur_nghb < nb_neighbour; cur_nghb++) {
        size_t neighbour = POPL_F->BLinkPoint(cur_node, cur_nghb);
        if (!marked[neighbour]) {
	  marked[neighbour] = true;
    }
  }



  size_t done = 0; // the number of nodes agglomerated in current level

  // going through all the nodes to agglomerate
  size_t cur_node = PL_F->begin();
  while (done < PL_F->NumEntries()) {

    // a node to agglomerate ?
    if (!marked[cur_node]) {
      marked[cur_node] = true;

      // create a new coarse node
      size_t cur_agglo = PL->AddEntry();
      PL->SetIVol(cur_agglo, PL_F->Vol(cur_node));

      // this is bogus - will be changed !!!
      PL->x(cur_agglo) = PL_F->x(cur_node);
      PL->y(cur_agglo) = PL_F->y(cur_node);
      PL->code(cur_agglo) = PL_F->code(cur_node);

      // a marker to check if the point has been added to BPL
      bool cur_agglo_in_BPL = false;
      if (PL->code(cur_agglo) != 0) {
	BPL->AddPoint(cur_agglo);
	cur_agglo_in_BPL = true;
      };

      mg_agent->SetCoarsePoint(cur_node, cur_agglo);
      size_t agglo_size = done;
      done++;

      // agglomerate all free neighbours
      size_t nb_neighbour = POPL_F->NumFLinks(cur_node);
      for (size_t cur_nghb = 0; cur_nghb < nb_neighbour; cur_nghb++) {
        size_t neighbour = POPL_F->FLinkPoint(cur_node, cur_nghb);
        if (!marked[neighbour]) {
	  marked[neighbour] = true;

          // a free node, get it !
	  mg_agent->SetCoarsePoint(neighbour, cur_agglo);
	  real vol = PL->IVol(cur_agglo);
	  vol += PL_F->Vol(neighbour);
	  PL->SetIVol(cur_agglo, vol);

	  // this is bogus - will be changed !!!
	  if (PL_F->code(neighbour) != 0) {
	    PL->x(cur_agglo) = PL_F->x(neighbour);
	    PL->y(cur_agglo) = PL_F->y(neighbour);
	    PL->code(cur_agglo) = PL_F->code(neighbour);

	    // add the new point in the BoundaryList
	    // if it is not already in there
	    if (!cur_agglo_in_BPL) {
	      BPL->AddPoint(cur_agglo);
	      cur_agglo_in_BPL = true;
	    };
	    
	    // set the normal vector (might change later ...)
	    BPL->SetNx(cur_agglo, BPL_F->Nx(neighbour));
	    BPL->SetNy(cur_agglo, BPL_F->Ny(neighbour));
	  };

          done++;
        };
      };
      agglo_size -= done;
    };

    // restart with another node !!! stupid choice can be better
    cur_node = PL_F->next(cur_node);
  };

  // inverse the volume
  FORALL(i, PL->) PL->SetIVol(i, PL->Vol(i));
};

