// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik
// Universitaet Duisburg, Germany
// Institute for Combustion and Gas Dynamics
// University of Duisburg, Germany
// Wed May 5 1999
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!
#include "AggloShape2D.hh"
#ifndef AggloShape2D_CC
#define AggloShape2D_CC

template <class POP>
AggloShape2D<POP>::AggloShape2D(MultiGridAgent<POP> *an_mga)
{
  mga = an_mga;
  POPL = mga->FinePOP();
  fcell = new CellProjection2D();
  fmm = NULL;
};

template <class POP>
void AggloShape2D<POP>::Set(TList<size_t> *points)
{
  fcell->Reset();
  FORALL(i, points->) {
    size_t p1 = points->At(i);
    size_t N = POPL->NumFLinks(p1);
    for(size_t j = 0; j < N; j++) {
      size_t p2 = POPL->FLinkPoint(p1,j);
      
      // check if the link is within the set 'points'
      try {
	points->FindItem(p2);
      } catch (NotFound_error) {
	// add the field vector if it is not
	fcell->AddVector(POPL->Fnx(p1,j), POPL->Fny(p1,j));
      };
    };

    if (mga != mga->Finest()) {
      // get a list of all boundary points for the field node
      TList<size_t> *b_nodes = mga->BoundaryPoints(p1);
            
      // loop over all this boundary nodes
      if (b_nodes != NULL) {
	FORALL(j, b_nodes->) {
	  size_t pb1 = b_nodes->At(j);
	  size_t N = POPL->NumBLinks(pb1);
	  for(size_t k = 0; k < N; k++) {
	    fcell->AddVector(POPL->Bnx(pb1,k), POPL->Bny(pb1,k));
	  };
	};
      };
    } else {
      // simply add all boundary links
      size_t N = POPL->NumBLinks(p1);
      for(size_t j = 0; j < N; j++) {
	fcell->AddVector(POPL->Bnx(p1,j), POPL->Bny(p1,j));
      };
    };
  };

  delete fmm;
  //ofstream f("fmm.dat");
  fmm = new FindMinMax<CellProjection2D>(fcell, 0 , M_PI, 10, 0.005);
  //exit(-1);
};

template <class POP>
AggloShape2D<POP>::~AggloShape2D()
{
  delete fmm;
  delete fcell;
};

#endif
