// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik
// Universitaet Duisburg, Germany
// Institute for Combustion and Gas Dynamics
// University of Duisburg, Germany
// Fri May 21 1999
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#include "CalcChDist2D.hh"

#ifndef CalcChDist2D_CC
#define CalcChDist2D_CC

template <class POP>
CalcChDist2D<POP>::CalcChDist2D(carg_t arg)
  : Action(arg)
{
  AddClassName();
  POPL = FindObject<POP>(false, true);
  PL = FindObject<PointList>(false, true);
};

template <class POP>
CalcChDist2D<POP>::~CalcChDist2D()
{
};

template <class POP>
void CalcChDist2D<POP>::Act()
{
  FORALL(i_point, PL->) {

    // get all neighbours of i_point
    size_t N = POPL->NumFLinks(i_point);
    if (N == 0) break;
    TList<real> alpha(5,5);
    TList<real> x(&alpha);
    TList<real> y(&alpha);
    TList<size_t> p(&alpha);
    for (size_t i_neigh = 0; i_neigh < N; i_neigh++) {
      size_t i_new = alpha.AddEntry();
      x[i_new] = PL->x(POPL->FLinkPoint(i_point, i_neigh));
      y[i_new] = PL->y(POPL->FLinkPoint(i_point, i_neigh));
      alpha[i_new] = Angle(1, 0, x[i_new]-PL->x(i_point), y[i_new]-PL->y(i_point));
      p[i_new] = POPL->FLinkPoint(i_point, i_neigh);
    };

    // sort the neighbours
    for (size_t i = alpha.last(); i > alpha.begin(); i = alpha.prev(i)) {
      for (size_t j = alpha.begin(); j < alpha.last(); j = alpha.next(j)) {
	if (alpha[j] > alpha[alpha.next(j)]) {
	  alpha.Swap(j, alpha.next(j));
	  x.Swap(j, alpha.next(j));
	  y.Swap(j, alpha.next(j));
	  p.Swap(j, alpha.next(j));
	};
      };
    };

    // put the first entry as additional entry at the end
    size_t i_new = alpha.AddEntry();
    x[i_new] = x[alpha.begin()];
    y[i_new] = y[alpha.begin()];
    alpha[i_new] = alpha[alpha.begin()];
    p[i_new] = p[alpha.begin()];

    // compute the minimum height of the triangles
    bool first = true;
    real h_min;
    for (size_t i = alpha.begin(); i < alpha.last(); i = alpha.next(i)) {
      try {
	//cout << p[i] << ' ' << p[p.next(i)] << endl;
	POPL->FindFLink(p[i], p[p.next(i)]);
	real beta = Angle(PL->x(i_point)-x[i], PL->y(i_point)-y[i], x[i+1]-x[i], y[i+1]-y[i]); 
	real b = DIST(PL->x(i_point), PL->y(i_point), x[i], y[i]);
	real h = fabs(b*sin(beta));
	if (first) {
	  first = false;
	  h_min = h;
	};
	if (h < h_min) h_min = h;
      } catch (NotFound_error) {};
    };

    // set the characteristic length to h_min
    PL->SetChDist(i_point, .5*h_min);
    //cout << endl;
  };
};

#endif

