// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#include <CppConstructArg.hh>
#include <CoarseGridToBoundary.hh>
#include <CoarseGridFromBoundary.hh>
#include <TBPOperation.hh>

template<class POP>
CalcGrad2D<POP>::Grad2DOperator::Grad2DOperator(carg_t arg) 
  : Operator(arg)
{
  AddClassName();
  VL = FindObject<VarList>(false, true);
  gx = VL->Get2DVar("gradx");
  gy = VL->Get2DVar("grady");
  CalcGrad2D<POP> *calc_grad = FindObject<CalcGrad2D<POP> >(true, true);
  var = VL->Get2DVar(calc_grad->FieldName());
};

template<class POP>
inline void CalcGrad2D<POP>::SetGradToZero::operate(size_t il, size_t i)
{
  for (iv = 0; iv < VL->NumVars(); iv++) {
    gx[iv][i] = 0.0;
    gy[iv][i] = 0.0;
  };
};

template<class POP>
CalcGrad2D<POP>::DivideGradByVol::DivideGradByVol(carg_t arg) 
  : CalcGrad2D<POP>::Grad2DOperator(arg)
{
  AddClassName();
  PL = FindObject<PointList>(false, true);
};

template<class POP>
inline void CalcGrad2D<POP>::DivideGradByVol::operate(size_t il, size_t i)
{
  for (iv = 0; iv < VL->NumVars(); iv++) {
    gx[iv][i] *= PL->IVol(i);
    gy[iv][i] *= PL->IVol(i);
  };
};

template<class POP>
CalcGrad2D<POP>::CalcGrad2D(carg_t arg) : Action(arg)
{
  AddClassName();
  field_name = arg.P->ReadString();
  mga = LookForObject<MultiGridAgent<POP> >(false, true);
  if (mga == NULL) {
    is_coarse_grid = false;
  } else {
    if (mga == mga->Finest()) is_coarse_grid = false;
    else is_coarse_grid = true;
  };
  //cout << "is_coarse_grid=" << is_coarse_grid << endl;
  set_to_zero = new TPOperation<SetGradToZero,POP>(this);
  divide_by_vol = new TPOperation<DivideGradByVol,POP>(this);
  if (is_coarse_grid) {
    transfer_to_boundary = new TBPOperation<CoarseGridToBoundary<POP>,POP>
      (ccarg_t(this) + (ccarg_t(this) + (ccarg_t(this) + field_name)));
    transfer_to_boundary->InsertAllCodes();
    transfer_x_from_boundary = new TBPOperation<CoarseGridFromBoundary<POP>,POP>
      (ccarg_t(this) + (ccarg_t(this) + (ccarg_t(this) + string("gradx"))));
    transfer_x_from_boundary->InsertAllCodes();
    transfer_y_from_boundary = new TBPOperation<CoarseGridFromBoundary<POP>,POP>
      (ccarg_t(this) + (ccarg_t(this) + (ccarg_t(this) + string("grady"))));
    transfer_y_from_boundary->InsertAllCodes();
  } else {
    transfer_to_boundary = NULL;
    transfer_x_from_boundary = NULL;
    transfer_y_from_boundary = NULL;
  };
};

template<class POP>
void CalcGrad2D<POP>::ToBoundary() 
{ 
  if (is_coarse_grid) transfer_to_boundary->Act();
};

template<class POP>
void CalcGrad2D<POP>::FromBoundary() 
{ 
  if (is_coarse_grid) {
    transfer_x_from_boundary->Act();
    transfer_y_from_boundary->Act();
  };
};








