// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#ifndef CalcGrad2D_HH
#define CalcGrad2D_HH

template<class POP> class CalcGrad2D;

#include <Action.hh>
#include <Operator.hh>
#include <BPOperation.hh>
#include <TPOperation.hh>
#include <VarList.hh>
#include <MultiGridAgent.hh>

/** */
template<class POP>
class CalcGrad2D : public Action
{
protected:
  /** */
  class Grad2DOperator : public Operator
  {
  protected:
    /** */ uint iv;
    /** */ VarContainer::var2_t gx;
    /** */ VarContainer::var2_t gy;
    /** */ VarContainer::var2_t var;
  public:
    /** */ Grad2DOperator(carg_t arg);
  };

  /** */
  class SetGradToZero : public Grad2DOperator
  {
  public:
    /** */ SetGradToZero(carg_t arg) : Grad2DOperator(arg) { AddClassName(); };
    /** */ void operate(size_t il, size_t i);
  };

  /** */
  class DivideGradByVol : public Grad2DOperator
  {
  protected:
    /** */ PointList *PL;
  public:
    /** */ DivideGradByVol(carg_t arg);
    /** */ void operate(size_t il, size_t i);
  };

  /** */ string field_name;
  /** */ Action *set_to_zero;
  /** */ Action *divide_by_vol;
  /** */ BPOperation *transfer_to_boundary;
  /** */ BPOperation *transfer_x_from_boundary;
  /** */ BPOperation *transfer_y_from_boundary;
  /** */ MultiGridAgent<POP> *mga;
  /** */ bool is_coarse_grid;
  /** */ void SetToZero() { set_to_zero->Act(); };
  /** */ void DivideByVol() { divide_by_vol->Act(); };
  /** */ void ToBoundary();
  /** */ void FromBoundary();

public:
  /** */ CalcGrad2D(carg_t arg);
  /** */ string FieldName() { return field_name; };
};

#include "CalcGrad2D.cc"

#endif







