// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik
// Universitaet Duisburg, Germany
// Institute for Combustion and Gas Dynamics
// University of Duisburg, Germany
// Tue Jun 8 1999
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!
#ifndef DualTimeStepCorrection_HH
#define DualTimeStepCorrection_HH

#include <mouse_config.h>

template <class POP> class DualTimeStepCorrection;

#include <mouse_types.hh>
#include <ConstructArg.hh>
#include <Operator.hh>
#include <CflManager.hh>
#include <MgAllLevelCflManager.hh>

/**
*/
template <class POP>
class DualTimeStepCorrection : public Operator
{
protected:
  /** */ VarContainer::var2_t Q_n;
  /** */ VarContainer::var2_t Q_k;
  /** */ VarContainer::var2_t res;
  /** */ MgAllLevelCflManager<POP> *CFLM_IMPL;

public:
  /** */ DualTimeStepCorrection (carg_t arg);
  /** */ virtual ~DualTimeStepCorrection ();
  /** */ void operate(size_t i_tmp, size_t i);
};

template <class POP>
inline void DualTimeStepCorrection<POP>::operate(size_t i_tmp, size_t i)
{
  real dt = CFLM_IMPL->CFL() * CFLM_IMPL->TimeStep(i);
  real V = PL->Vol(i);
  //cout << dt << endl;
  for (size_t iv = 0; iv < VL->NumVars(); iv++) {
    //res[iv][i] = (res[iv][i]*dt - Q[iv][i]*V) / (dt+dtau);
    res[iv][i] += V*(Q_n[iv][i]-Q_k[iv][i])/dt;
  };
};


#include "DualTimeStepCorrection.cc"

#endif
