// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik
// Universitaet Duisburg, Germany
// Institute for Combustion and Gas Dynamics
// University of Duisburg, Germany
// Tue Jun 1 1999
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!
#ifndef EulerProlongation2D_HH
#define EulerProlongation2D_HH

#include <mouse_config.h>

template <class POP> class EulerProlongation2D;

#include <mouse_types.hh>
#include <ConstructArg.hh>
#include <MultiGridTransferOperator.hh>
#include <BoundaryPointList.hh>
#include <geometry2D.hh>

/**
*/
template <class POP>
class EulerProlongation2D : public MultiGridTransferOperator<POP>
{
protected:
  /** */ BoundaryPointList *BPL_C;
  /** */ VarContainer::var1_t rho_corr;
  /** */ VarContainer::var1_t rhou_corr;
  /** */ VarContainer::var1_t rhov_corr;
  /** */ VarContainer::var1_t rhoE_corr;
  /** */ VarContainer::var1_t rho_C;
  /** */ VarContainer::var1_t rhou_C;
  /** */ VarContainer::var1_t rhov_C;
  /** */ VarContainer::var1_t rhoE_C;
  /** */ VarContainer::var1_t rho_F;
  /** */ VarContainer::var1_t rhou_F;
  /** */ VarContainer::var1_t rhov_F;
  /** */ VarContainer::var1_t rhoE_F;
  /** */ MIntList *codes;

public:
  /** */ EulerProlongation2D (carg_t arg);
  /** */ virtual ~EulerProlongation2D ();
  /** */ void operate(size_t i_tmp, size_t i_fine);
};

template<class POP>
inline void EulerProlongation2D<POP>::operate(size_t i_tmp, size_t i_fine)
{
  if (TransferPoint(i_fine)) {
    size_t i_coarse = mg_agent->CoarsePoint(i_fine);
    bool is_twall = false;
    FORALL(i, codes->) {
      if (PL_coarse->code(i_coarse) == codes->At(i)) {
	is_twall = true;
	break;
      };
    };
    if (is_twall) {
      return;
      if (PL_fine->code(i_fine) != 0) {
	// coarse grid correction tangential vector
	real tx_corr = rhou_corr[i_coarse];
	real ty_corr = rhov_corr[i_coarse];
	real tabs_corr = DIST(0, 0, tx_corr, ty_corr);
	
	// initial coarse grid tangential vector
	real tx_C = rhou_C[i_coarse];
	real ty_C = rhov_C[i_coarse];
	
	if (tx_corr*tx_C + ty_corr*ty_C < 0) tabs_corr *= -1;
	
	// fine grid "tangential" vector
	real tx_F = rhou_F[i_fine];
	real ty_F = rhov_F[i_fine];
	real tabs_F = DIST(0, 0, tx_F, ty_F);
	tx_F /= tabs_F;
	ty_F /= tabs_F;
	
	// vector correction for the momentum
	rhou_F[i_fine] += factor * tx_F * tabs_corr;
	rhov_F[i_fine] += factor * ty_F * tabs_corr;
	
	// standard correction for density and energy
	rho_F[i_fine] += factor * rho_corr[i_coarse];
	rhoE_F[i_fine] += factor * rhoE_corr[i_coarse];
      };
    } else {
      // standard correction for non twall nodes
      rho_F[i_fine] += factor * rho_corr[i_coarse];
      rhou_F[i_fine] += factor * rhou_corr[i_coarse];
      rhov_F[i_fine] += factor * rhov_corr[i_coarse];
      rhoE_F[i_fine] += factor * rhoE_corr[i_coarse];
    };
  };  
};


#include "EulerProlongation2D.cc"

#endif
