// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik
// Universitaet Duisburg, Germany
// Institute for Combustion and Gas Dynamics
// University of Duisburg, Germany
// Thu Jun 3 1999
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!
#ifndef EulerWall2D_HH
#define EulerWall2D_HH

#include <mouse_config.h>

template <class POP> class EulerWall2D;

#include <mouse_types.hh>
#include <ConstructArg.hh>
#include <CompressibleBPOperator2D.hh>

/**
*/
template <class POP>
class EulerWall2D : public CompressibleBPOperator2D
{
protected:
  /** */ POP *POPL;

public:
  /** */ EulerWall2D (carg_t arg);
  /** */ virtual ~EulerWall2D ();
  /** */ void operate(size_t i_tmp, size_t i);
};

template <class POP>
inline void EulerWall2D<POP>::operate(size_t i_tmp, size_t i)
{
  // tangential correction
  real rhouv = rhou[i] * BPL->Nx(i) + rhov[i] * BPL->Ny(i);
  rhou[i] = rhou[i] - rhouv * BPL->Nx(i);
  rhov[i] = rhov[i] - rhouv * BPL->Ny(i);

  //   dH/dn = 0
  // & dS/dn = 0
  size_t N = 0;
  real H = 0;
  real P = 0;
  for (size_t j = 0; j < POPL->NumFLinks(i); j++) {
    size_t jp = POPL->FLinkPoint(i,j);
    if (PL->code(jp) == 0) {
      real p = (gamma-1)*(rhoE[jp]-.5/rho[jp]*(rhou[jp]*rhou[jp]+rhov[jp]*rhov[jp]));
      H += (rhoE[jp] + p) / rho[jp];
      P += p;
      N++;
    };
  };
  H /= N;
  P /= N;
  rhoE[i] = rho[i]*H - P;
};

#include "EulerWall2D.cc"

#endif
