template<class F>
FindMinMax<F>::FindMinMax(F *an_f, real x_min, real an_x_max, int N, real a_prec)
{
  steps = 0;
  f = an_f;
  x_max = an_x_max;
  delta_x = (x_max-x_min)/N;
  d_x = delta_x;
  prec = a_prec;
  real f_xmin = f->calc(x_min);
  real f_xmax = f->calc(x_max);
  if (f_xmin > f_xmax) {
    f_min = f_xmax;
    f_max = f_xmin;
    fx_min = x_max;
    fx_max = x_min;
  } else {
    f_max = f_xmax;
    f_min = f_xmin;
    fx_min = x_min;
    fx_max = x_max;
  };
  while(Advance(x_min));
};

template<class F>
real FindMinMax<F>::ComputeValues(real x0, real x1, real x2, 
				  real &f0, real &f1, real &f2)
{
  f0 = f->calc(x0);
  f1 = f->calc(x1);
  f2 = f->calc(x2);
  real a = .5*DIST(x0,f0,x2,f2);
  real c = DIST(x1,f1,x2,f2);
  return sqrt(SQR(c)-SQR(a));
};

template<class F>
bool FindMinMax<F>::Advance(real &x0)
{
  bool advanced = true;

  // get a first guess for the values
  steps++;
  real x2 = x0 + d_x;
  if (x2 > x_max) {
    x2 = x_max;
    advanced = false;
  };
  real x1 = .5*(x0 + x2);

  // compute function values and distance factor
  real f0,f1,f2;
  real dist = ComputeValues(x0,x1,x2,f0,f1,f2);

  // check if we can increase the step size
  if (dist < .8*prec) {
    steps++;
    d_x *= prec/dist;
    x2 = x0 + d_x;
    if (x2 > x_max) {
      x2 = x_max;
      advanced = false;
    };
    x1 = .5*(x0 + x2);

    // compute function values and distance factor
    dist = ComputeValues(x0,x1,x2,f0,f1,f2);
  };

  // decrease step width if necessary
  while (dist > prec) {
    steps++;
    x2 = x1;
    x1 = .5*(x0 + x2);

    // compute function values and distance factor
    dist = ComputeValues(x0,x1,x2,f0,f1,f2);

    advanced = true;
  };

  if (f1 < f_min) {
    f_min = f1;
    fx_min = x1;
  };
  if (f1 > f_max) {
    f_max = f1;
    fx_max = x1;
  };
  x0 = x1;
  //cout << x1 << " " << f1 << endl;
  return advanced;
};


