// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik
// Universitaet Duisburg, Germany
// Institute for Combustion and Gas Dynamics
// University of Duisburg, Germany
// Tue Aug 24 1999
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#include "MergeGrid2D.hh"

#ifndef MergeGrid2D_CC
#define MergeGrid2D_CC

template <class POP>
MergeGrid2D<POP>::MergeGrid2D(carg_t arg)
  : Action(arg)
{
  AddClassName();
  PL = FindObject<PointList>(false, true);
  EL = FindObject<ElementList>(false, true);
  POPL = FindObject<POP>(false, true);
  BPL = FindObject<BoundaryPointList>(false, true);
  merge_obj = ResolveVar<MObject>(arg);
  merge_obj->FindObject(PL_merge, false, true);
  merge_obj->FindObject(EL_merge, false, true);
  merge_obj->FindObject(POPL_merge, false, true);
  merge_obj->FindObject(BPL_merge, false, true);
};

template <class POP>
MergeGrid2D<POP>::~MergeGrid2D()
{
};

// compute the minimal boundary edge  length
template <class POP>
real MergeGrid2D<POP>::MinEdge(PointList &PL, POP &POPL)
{
  real l_min = 1e20;
  FORALL(i, PL.) {
    for (size_t j = 0; j < POPL.NumFLinks(i); j++) {
      size_t n = POPL.FLinkPoint(i, j);
      real l = DIST(PL.x(i), PL.y(i), PL.x(n), PL.y(n));
      if (l < l_min) l_min = l;
    };
  };
  return l_min;
};

template <class POP>
void MergeGrid2D<POP>::Act()
{
  TList<size_t> here_points(100, 100);
  TList<size_t> there_points(&here_points);
  real l_here = MinEdge(*PL, *POPL);
  real l_there = MinEdge(*PL_merge, *POPL_merge);
  real l;
  if (l_here < l_there) l = l_here;
  else l = l_there;
  l *= .1;
  //cout << "l=" << l << endl;

  // find common points
  // algorithm is O(N^2) => future improvement necessary
  FORALL(i, PL->) {
    size_t p_here = i;//BPL->FIndex(i);
    FORALL(j, PL_merge->) {
      size_t p_there = j;//BPL_merge->FIndex(j);
      real d = DIST(PL->x(p_here), PL->y(p_here),
		    PL_merge->x(p_there), PL_merge->y(p_there));
      //cout << p_there << ',' << p_here << "  d=" << d << endl;
      if (d < l) {
	// p_here and p_there are common to both grids
	size_t np = here_points.AddEntry();
	here_points[np] = p_here;
	there_points[np] = p_there;
      };
    };
  };

  // mark 'there' points which are common
  TList<bool> there_common(PL_merge, false);
  FORALL(i, there_points.) there_common[there_points[i]] = true;

  // add 'there' points and memorize mapping
  TList<size_t> new_index(PL_merge);
  FORALL(i, PL_merge->) {
    if (!there_common[i]) {
      size_t np = PL->AddEntry();
      PL->x(np) = PL_merge->x(i);
      PL->y(np) = PL_merge->y(i);
      PL->code(np) = PL_merge->code(i);
      new_index[i] = np;
    };
  };

  // correct mapping
  FORALL(i, there_points.) {
    new_index[there_points[i]] = here_points[i];
  };

  //debug!!
//   FORALL(i, PL_merge->) cout << i << ' ' << new_index[i] << endl;
//   FORALL(i, there_points.) {
//     cout << "here:" << here_points[i] << " there:" << there_points[i] << endl;
//   };

  // add 'there' elements and update indices
  FORALL(i, EL_merge->) {
    size_t nn = EL_merge->NumNodes(i);
    size_t nodes[nn];
    for (size_t n = 0; n < nn; n++) {
      nodes[n] = new_index[EL_merge->Node(i, n)];
    };
    EL->AddElement(nn, nodes);
  };
};

#endif

