// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik
// Universitaet Duisburg, Germany
// Institute for Combustion and Gas Dynamics
// University of Duisburg, Germany
// Mon Jun 21 1999
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#include "MgAllLevelCflManager.hh"

#ifndef MgAllLevelCflManager_CC
#define MgAllLevelCflManager_CC

#include <PointList.hh>

template <class POP>
MgAllLevelCflManager<POP>::MgAllLevelCflManager(carg_t arg)
  : Action(arg)
{
  AddClassName();
  mga = FindObject<MultiGridAgent<POP> >(false, true);
  PointList *PL = FindObject<PointList>(false, true);
  if (mga != mga->Finest()) {
    // for coarse levels
    //
    mga->Prev()->FindObject(CFLM_fine, false, true);
    CFLM_fine->CFLM_coarse = this;
    REAL_CFLM = NULL;
    delta_t = new TList<real>(PL,0);
    // create the restriction on the previous level !!
    // the parent object in the tree is mga->Prev()
    restrict = new TPOperation<RestrictTimeStep,POP>(mga->Prev());
  } else {
    // for the finest level
    //
    CFLM_fine = NULL;
    REAL_CFLM = BuildObject<CflManager>(arg);
    delta_t = REAL_CFLM->DeltaT();
  };
  CFLM_coarse = NULL;
};

template <class POP>
MgAllLevelCflManager<POP>::~MgAllLevelCflManager()
{
  // make sure the parent class does not delete bogus (SIGSEGV!)
  if (mga == mga->Finest()) delta_t = NULL;
  delete REAL_CFLM;
};

template <class POP>
void MgAllLevelCflManager<POP>::Act()
{
  if (mga != mga->Finest()) {
    // for coarse levels
    cfl = CFLM_fine->CFL();
    FORALL(i, delta_t->) delta_t->At(i) = 0;
    restrict->Act();
  } else {
    // for the finest level
    REAL_CFLM->Act();
  };
};

template <class POP>
MgAllLevelCflManager<POP>::RestrictTimeStep::RestrictTimeStep(carg_t arg)
  : Operator(arg)
{
  AddClassName();
  mga = FindObject<MultiGridAgent<POP> >(false, true);
  CFLM_fine = FindObject<MgAllLevelCflManager<POP> >(false, true);
  CFLM_coarse = CFLM_fine->CoarseCfl();
  PL_fine = mga->FinePL();
  PL_coarse = mga->CoarsePL();
  if (CFLM_fine == NULL) Bug();
  if (CFLM_coarse == NULL) Bug();
};


#endif

