// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik
// Universitaet Duisburg, Germany
// Institute for Combustion and Gas Dynamics
// University of Duisburg, Germany
// Mon Jun 21 1999
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!
#ifndef MgAllLevelCflManager_HH
#define MgAllLevelCflManager_HH

#include <mouse_config.h>

template <class POP> class MgAllLevelCflManager;

#include <mouse_types.hh>
#include <ConstructArg.hh>
#include <CflManager.hh>
#include <TPOperation.hh>
#include <MultiGridAgent.hh>
#include <Operator.hh>

/**
   Provides information of a CflManager, defined on the finest level, to all multi-grid levels.
   This class is used for the dual-time-stepping method.
*/
template <class POP>
class MgAllLevelCflManager : public Action
{
protected:
  /** used to restrict delta_t */
  struct RestrictTimeStep : public Operator {
    /** */ MgAllLevelCflManager<POP> *CFLM_fine;
    /** */ MgAllLevelCflManager<POP> *CFLM_coarse;
    /** */ MultiGridAgent<POP> *mga;
    /** */ PointList *PL_fine;
    /** */ PointList *PL_coarse;
    /** */ RestrictTimeStep(carg_t arg);
    /** */ void operate(size_t i_tmp, size_t i);
  };

  /** agent of this level */
  MultiGridAgent<POP> *mga;

  /** the next finer level CflManger (NULL for finest levels) */ 
  MgAllLevelCflManager<POP> *CFLM_fine;

  /** the next coarser level CflManger (NULL for coarsest levels) */ 
  MgAllLevelCflManager<POP> *CFLM_coarse;

  /** CflManager doing the real work (NULL for coarse levels) */
  CflManager *REAL_CFLM;

  /** used to restrict the time steps from the next finer level */
  TPOperation<RestrictTimeStep,POP> *restrict;

  /** list for the time step width */ 
  TList<real> *delta_t;

  /** the CFL number (only stored if REAL_CFLM == NULL) */
  real cfl;

public:
  /** */ MgAllLevelCflManager (carg_t arg);
  /** */ virtual ~MgAllLevelCflManager ();
  
  /**
     get the next finer level CflManger
     @return pointer to the next finer level CflManger
  */
   MgAllLevelCflManager<POP>* FineCfl() { return CFLM_fine; };
  
  /**
     get the next coarser level CflManger
     @return pointer to the next finer level CflManger
  */
  MgAllLevelCflManager<POP>* CoarseCfl() { return CFLM_coarse; };

  /** */ virtual void Act();
  /** */ real TimeStep(size_t i) { return delta_t->At(i); };
  /** */ void SetTimeStep(size_t i, real dt) { delta_t->At(i) = dt; };
  /** */ real CFL();
};


template <class POP>
inline void MgAllLevelCflManager<POP>::RestrictTimeStep::operate(size_t i_tmp, size_t i)
{
  if (!mga->IsHiddenNode(i)) {
    size_t i_coarse = mga->CoarsePoint(i);
    real dt_coarse = CFLM_coarse->TimeStep(i_coarse);
    real dt_fine = CFLM_fine->TimeStep(i);
    CFLM_coarse->SetTimeStep(i_coarse, dt_coarse 
			     + dt_fine * PL_fine->Vol(i) * PL_coarse->IVol(i_coarse));
    //CFLM_coarse->SetTimeStep(mga->CoarsePoint(i), CFLM_fine->TimeStep(i));
  };
};

template <class POP>
inline real MgAllLevelCflManager<POP>::CFL()
{ 
  if (FineCfl() == NULL) return REAL_CFLM->CFL();
  else return cfl;
};

#include "MgAllLevelCflManager.cc"

#endif
