// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik
// Universitaet Duisburg, Germany
// Institute for Combustion and Gas Dynamics
// University of Duisburg, Germany
// Fri May 7 1999
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!
#ifndef MultiGridAveragedProlongation_HH
#define MultiGridAveragedProlongation_HH

#include <mouse_config.h>

template <class POP> class MultiGridAveragedProlongation;

#include <mouse_types.hh>
#include <ConstructArg.hh>
#include <MultiGridTransferOperator.hh>

/**
*/
template <class POP>
class MultiGridAveragedProlongation : public MultiGridTransferOperator<POP>
{
protected:
  /** */ POP *POPL;
  /** */ VarContainer::var2_t gradx_C;
  /** */ VarContainer::var2_t grady_C;
  /** */ VarContainer::var2_t gradx_F;
  /** */ VarContainer::var2_t grady_F;
  /** */ VarContainer::var2_t damp;
  /** */ real K;

public:
  /** */ MultiGridAveragedProlongation(carg_t arg);
  /** */ virtual ~MultiGridAveragedProlongation();
  /** */ void operate(size_t i_tmp, size_t i_fine);
};

template<class POP>
inline void MultiGridAveragedProlongation<POP>::operate(size_t i_tmp, size_t i_fine)
{
  if (TransferPoint(i_fine)) {
    size_t i_coarse = mg_agent->CoarsePoint(i_fine);
    real vx = PL->x(i_fine) - PL_coarse->x(i_coarse);
    real vy = PL->y(i_fine) - PL_coarse->y(i_coarse);
    for (size_t i_var = 0; i_var < VL->NumVars(); i_var++) {
      real gradx = gradx_C[i_var][i_coarse];
      real grady = grady_C[i_var][i_coarse];
      real corr = var_coarse[i_var][i_coarse] + (gradx * vx + grady * vy);
      var_fine[i_var][i_fine] += factor * corr;
    };
  };

  /*
  if (TransferPoint(i_fine)) {
    size_t N = POPL->NumFLinks(i_fine);
    for (size_t i_var = 0; i_var < VL->NumVars(); i_var++) {
      real corr = 0;
      //real tot_vol = 0;
      for (size_t i_neigh = 0; i_neigh < N; i_neigh++) {
	size_t neigh_fine = POPL->FLinkPoint(i_fine, i_neigh);
	size_t i_coarse = mg_agent->CoarsePoint(neigh_fine);
	//corr += PL_fine->Vol(neigh_fine)*var_coarse[i_var][i_coarse];
	corr += var_coarse[i_var][i_coarse];
	//tot_vol += PL_fine->Vol(neigh_fine);
      };
      size_t i_coarse = mg_agent->CoarsePoint(i_fine);
      //corr += PL_fine->Vol(i_fine)*var_coarse[i_var][i_coarse];
      //tot_vol += PL_fine->Vol(i_fine);
      //corr /= tot_vol;
      corr /= N;
      var_fine[i_var][i_fine] += factor * (K*corr + (1-K)*var_coarse[i_var][i_coarse]);
    };
  };
  */

  /*
  if (TransferPoint(i_fine)) {
    size_t i_coarse = mg_agent->CoarsePoint(i_fine);
    for (size_t i_var = 0; i_var < VL->NumVars(); i_var++) {
      real corr = var_coarse[i_var][i_coarse];
      var_fine[i_var][i_fine] += factor * corr;
      damp[i_var][i_fine] = fabs(corr);
    };
  };
  */
};

#include "MultiGridAveragedProlongation.cc"

#endif
