#ifndef MultiGridTransferOperator_HH
#define MultiGridTransferOperator_HH

template<class POP> class MultiGridTransferOperator;

#include <Operator.hh>
#include <MultiGridAgent.hh>

/**
   a transfer operator between two grid levels in a multi-grid computation.
*/
template<class POP>
class MultiGridTransferOperator : public Operator
{
protected:
  /**
     the agent administrating this grid level
  */
  MultiGridAgent<POP> *mg_agent;

  /**
     the fine grid's VarList
  */
  VarList *VL_fine;

  /**
     the fine grid's PointList
  */
  PointList *PL_fine;

  /**
     the coarse grid's VarList
  */
  VarList *VL_coarse;

  /**
     the coarse grid's PointList
  */
  PointList *PL_coarse;

  /**
     the fine grid name of the field to transfer
  */
  string fine_field_name;

  /**
     the coarse grid name of the field to transfer
  */
  string coarse_field_name;

  /**
     a 2D variable to that field on the fine mesh
  */
  VarContainer::var2_t var_fine;

  /**
     a 2D variable to that field on the coarse mesh
  */
  VarContainer::var2_t var_coarse;

  /**
     a factor to multiply the transfered field by
  */
  real factor;

public:
  /** */ MultiGridTransferOperator(carg_t arg);

  /**
     check if a point has to be transferred
     @param i the fine grid index
     @return true if it has to be transferred
  */
  bool TransferPoint(size_t i) {
    /*
    if (PL_fine->code(i) == 0) return true;
    if (mg_agent == mg_agent->Finest()) return true;
    return false;
    */
    return !mg_agent->IsHiddenNode(i);
  };
};

#include "MultiGridTransferOperator.cc"

#endif
