// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik
// Universitaet Duisburg, Germany
// Institute for Combustion and Gas Dynamics
// University of Duisburg, Germany
// Mon Jun 14 1999
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!
#ifndef RungeKuttaDc_HH
#define RungeKuttaDc_HH

#include <mouse_config.h>

template <class POP> class RungeKuttaDc;

#include <mouse_types.hh>
#include <ConstructArg.hh>
#include <Operator.hh>
#include <RungeKutta.hh>

/**
*/
template <class POP>
class RungeKuttaDc : public RungeKutta<POP>
{
protected:
  /** */ 
  struct DcPartialStep : public SubOperator {
    DcPartialStep(carg_t arg);
    void operate(size_t il, size_t);
  };

  /** */ TPOperation<RungeKuttaDc<POP>::DcPartialStep, POP> *partial_dc_step;
  /** */ virtual void Partial() { partial_dc_step->Act(); };

public:
  /** */ RungeKuttaDc (carg_t arg);
  /** */ virtual ~RungeKuttaDc ();
};

template<class POP>
inline void RungeKuttaDc<POP>::DcPartialStep::operate(size_t il, size_t i)
{
  real RES;
  for (ivar = 0; ivar < VL->NumVars(); ivar++) {
    RES = 2*var_res[ivar][i];
    var_new[ivar][i] = var_old[ivar][i] 
      + tstep->Factor() * CFLM->CFL() * CFLM->TimeStep(i) * PL->IVol(i) * RES;
  };
};


#include "RungeKuttaDc.cc"

#endif
