// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik
// Universitaet Duisburg, Germany
// Institute for Combustion and Gas Dynamics
// University of Duisburg, Germany
// Mon Jun 7 1999
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!
#ifndef SmallSquareMatrix_HH
#define SmallSquareMatrix_HH

#include <mouse_config.h>

template <class T, uint N> class SmallSquareMatrix;

#include <mouse_types.hh>
#include <ConstructArg.hh>
#include <SmallVector.hh>

/**
*/
template <class T, uint N>
class SmallSquareMatrix : public SmallVector<SmallVector<T,N>,N>
{
protected:
  /** */ T limit;

public:
  /** */ SmallSquareMatrix<T,N>(T a_limit = 1e-20) : SmallVector<SmallVector<T,N>,N>() { limit = a_limit; };
  /** */ SmallSquareMatrix<T,N>(const SmallSquareMatrix<T,N> &M) : SmallVector<SmallVector<T,N>,N>(M) {};
  /** */ static SmallSquareMatrix<T,N> Identity();

  /** */ 
  void MultRow(uint r, T s) { 
    for(uint i=0; i<N; i++) V[r][i] *= s; 
  };

  /** */ 
  void SubstractRow(uint r1, uint r2) {
    for(uint i=0; i<N; i++) V[r2][i] -= V[r1][i]; 
  };

  /** */ void SwapRows(uint r1, uint r2);
  /** */ uint FindNonZero(uint c);
  /** */ uint NumLowerZeros(uint r);
  /** */ uint NumUpperZeros(uint r);
  /** */ void Sort(SmallSquareMatrix<T,N> &M);
  /** */ void Conditionate(SmallSquareMatrix<T,N> &M);
  /** */ SmallSquareMatrix<T,N> Inv();

  /** */ T AbsMax();
};


#include "SmallSquareMatrix.cc"

#endif
