// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik
// Universitaet Duisburg, Germany
// Institute for Combustion and Gas Dynamics
// University of Duisburg, Germany
// Tue Jun 15 1999
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!
#ifndef SmoothingStep_HH
#define SmoothingStep_HH

#include <mouse_config.h>

template <class POP> class SmoothingStep;

#include <mouse_types.hh>
#include <ConstructArg.hh>
#include <Operator.hh>
#include <TimeStep.hh>

/**
*/
template <class POP>
class SmoothingStep : public TimeStep<POP>
{
protected:
  /** */ 
  struct PartialSmoothingStep : public SubOperator {
    real damp;
    PartialSmoothingStep(carg_t arg) : SubOperator(arg) { AddClassName(); };
    void operate(size_t il, size_t);
  };

  /** */ uint N;
  /** */ Action *sub_smooth_step;
  /** */ virtual void Partial() { sub_smooth_step->Act(); };

public:
  /** */ SmoothingStep (carg_t arg);
  /** */ virtual ~SmoothingStep ();
  /** */ virtual void Act();
};

template<class POP>
inline void SmoothingStep<POP>::PartialSmoothingStep::operate(size_t il, size_t i)
{
  real dtau = CFLM->CFL() * CFLM->TimeStep(i);
  real dt = tstep->Factor() * CFLM->TimeStep(i);
  real C = dt*dtau/(dt+dtau);
  for (size_t iv = 0; iv < VL->NumVars(); iv++) {
    var_new[iv][i] = C*(PL->IVol(i)*var_res[iv][i] + var_old[iv][i]/dt + var_new[iv][i]/dtau);
  };
};

#include "SmoothingStep.cc"

#endif
