// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

//
//.. constructor
//
template<class OP, class POP>
TFEdgeCalculator2D<OP, POP>::TFEdgeCalculator2D(ConstructArg arg) 
  : PointCalculator(arg) 
{
  AddClassName();
  op = new OP(carg_t(this, arg.P->SubParser()));
  mesh = FindObject<POP>(false, true);
};

//
//.. CalcGroup
//
template<class OP, class POP>
void TFEdgeCalculator2D<OP, POP>::CalcGroup(size_t i_begin, size_t i_end) 
{
  size_t i_from, i_to;
  unsigned int j;
  unsigned int j_max;
  unsigned int gid_f = thread_id;
  unsigned int gid_t;
  bool mod_to;
  bool calc_edge;
  // it is assumed that the group_id of the from point always belongs
  // to this thread (the PointAction object should take care of that)
  bool mod_from;
  size_t group_index = 0;
  // check and eventually extend the temporary point memory
  if (i_end - i_begin > point_tmp->NumEntries()) {
    point_tmp->ExtendBy(i_end - i_begin - point_tmp->NumEntries());
  };
  for (i_from = i_begin; i_from < i_end; i_from = PL->next(i_from)) {
    j_max = mesh->NumFLinks(i_from);
    // check and eventually extend the temporary edge memory
    if (group_index + j_max > edge_tmp->NumEntries()) {
      edge_tmp->ExtendBy(group_index + j_max - edge_tmp->NumEntries());
    };
    for (j = 0; j < j_max; j++) {
      i_to = mesh->FLinkPoint(i_from, j);
      gid_t = mesh->GroupID(i_to);
      mod_to = (gid_f == gid_t);
      mod_from = true;
      calc_edge = (i_from < i_to);
      if (!calc_edge && !mod_to) calc_edge = true;
      if (calc_edge) op->operate(group_index, i_from - i_begin, i_from, i_to, 
				 mesh->Fnx(i_from, j), mesh->Fny(i_from, j),
				 mod_from, mod_to);
      group_index++;
    };	
  };
};













