// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#if defined(TLIST_H)

//
//.. constructor
//
template <class T>
TList<T>::TList (size_t a_max_num_entries, size_t a_delta_entries) 
  : List (a_max_num_entries, a_delta_entries)
{
  value = new T [max_num_entries];
};

template <class T>
TList<T>::TList (size_t a_max_num_entries, size_t a_delta_entries, T a_default_value) 
  : List (a_max_num_entries, a_delta_entries)
{
  value = new T [max_num_entries];
  default_value = a_default_value;
};

template <class T>
TList<T>::TList (List *a_master) : List (a_master)
{
  value = new T [max_num_entries];
  //value = new T [20000];//@@
};

template <class T>
TList<T>::TList (List *a_master, T a_default_value) : List (a_master)
{
  value = new T [max_num_entries];
  default_value = a_default_value;
  InitAllEntries(default_value);
};

template <class T>
TList<T>::TList ()
{
  value = NULL;
};

//
//.. destructor
//
template <class T>
TList<T>::~TList () {
  delete [] value;
};  

//
//.. InitAllEntries
//
template <class T>
void TList<T>::InitAllEntries (T a_value) {
  size_t i;
  for (i = 0; i < num_entries; i++) value[i] = a_value;
};

//
//.. Extend
//
template <class T>
void TList<T>::Extend (size_t delta) {
  T *new_value;
  size_t i_entry;
  new_value = new T [max_num_entries + delta];
  //new_value = new T[20000];//@@
  for (i_entry = 0; i_entry < num_entries; i_entry++) new_value[i_entry] = value[i_entry];
  //for (i_entry = 0; i_entry < 20000; i_entry++) new_value[i_entry] = value[i_entry];//@@
  delete [] value;
  value = new_value;
};

//
//.. FindItem
//
template <class T>
size_t TList<T>::FindItem (T item) const
{
  //size_t i;
  //_FORALL(i, this->) if (value[i] == item) return i;
  for(size_t i = begin(); i < end(); i = next(i)) {
    if (value[i] == item) return i;
  };
  throw NotFound_error();
};

//
//.. IsIn
//
template <class T>
bool TList<T>::Has(T item) const
{
  for(size_t i = begin(); i < end(); i = next(i)) {
    if (value[i] == item) return true;
  };
  return false;
};

//
//.. operator=
//
template <class T>
void TList<T>::operator=(const TList<T> &other)
{
  List::operator=(other);
  delete[] value;
  //size_t i;
  //_FORALL(i, other.) value[i] = other[i];
  for(size_t i = begin(); i < end(); i = next(i)) {
    value[i] = other[i];
  };
};


#endif







