// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#if defined(TLIST2D_H)

//
//.. constructor
//
template <class T>
TList2D<T>::TList2D (long int a_max_num_entries, long int a_delta_entries, int a_dim) 
  : List (a_max_num_entries, a_delta_entries)
{
  int i;
  dim = a_dim;
  value = new T* [dim];
  value[0] = new T [max_num_entries * dim];
  for (i = 1; i < dim; i++) value[i] = value[i-1] + max_num_entries;
};

template <class T>
TList2D<T>::TList2D (long int a_max_num_entries, long int a_delta_entries, 
		     int a_dim, T a_default_value) 
  : List (a_max_num_entries, a_delta_entries)
{
  int i;
  dim = a_dim;
  value = new T* [dim];
  value[0] = new T [max_num_entries * dim];
  for (i = 1; i < dim; i++) value[i] = value[i-1] + max_num_entries;
  default_value = a_default_value;
};

template <class T>
TList2D<T>::TList2D (int a_dim, List *a_master) : List (a_master) {
  int i;
  dim = a_dim;
  value = new T* [dim];
  value[0] = new T [max_num_entries * dim];
  for (i = 1; i < dim; i++) value[i] = value[i-1] + max_num_entries;
};  

template <class T>
TList2D<T>::TList2D (int a_dim, List *a_master, T a_default_value) : List (a_master) {
  int i;
  dim = a_dim;
  value = new T* [dim];
  value[0] = new T [max_num_entries * dim];
  for (i = 1; i < dim; i++) value[i] = value[i-1] + max_num_entries;
  default_value = a_default_value;
};  

//
//.. destructor
//
template <class T>
TList2D<T>::~TList2D () {
  delete [] value[0];
  delete [] value;
};  

//
//.. Extend
//
template <class T>
void TList2D<T>::Extend (long int delta) {
  int i;
  long int i_entry;
  T **new_value;
  new_value = new T* [dim];
  new_value[0] = new T [(max_num_entries + delta) * dim];
  for (i = 1; i < dim; i++) new_value[i] = new_value[i-1] + max_num_entries + delta;
  for (i = 0; i < dim; i++) {
    for (i_entry = 0; i_entry < num_entries; i_entry++) {
      new_value[i][i_entry] = value[i][i_entry];
    };
  };
  delete [] value[0];
  delete [] value;
  value = new_value;
};

//
//.. CopyEntry
//
template <class T>
void TList2D<T>::CopyEntry (long int src, long int dst) {
  int i;
  for (i = 0; i < dim; i++) At(i, dst) = At(i, src);
};

//
//.. InitEntry
//
template <class T>
void TList2D<T>::InitEntry (long int j) {
  int i;
  for (i = 0; i < dim; i++) At(i, j) = default_value;
};

#endif







