// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#ifdef TMDIMVAR_H
//
//.. constructor
//
template<class TValue, class TIndex, int DIM, class MAP, class MAPVALUE>
TMDimVar<TValue, TIndex, DIM, MAP, MAPVALUE>::TMDimVar
(TMDimList<TValue, TIndex, DIM, MAP> *a_mdim_list, TMDimIndex<TIndex> an_index)
  : TMappedVar<TValue, TIndex, DIM, MAP>(a_mdim_list, an_index)
{
  // check if index is legal
  if (index.Dim() > DIM) {
    cerr << "illegal index dimension for TMDimVar" << endl;
    exit(EXIT_FAILURE);
  };
  value = NULL;
  Update();
};

template<class TValue, class TIndex, int DIM, class MAP, class MAPVALUE>
TMDimVar<TValue, TIndex, DIM, MAP, MAPVALUE>::TMDimVar
(const TMDimVar<TValue, TIndex, DIM, MAP, MAPVALUE> &other)
  : TMappedVar<TValue, TIndex, DIM, MAP>(other.mdim_list, other.index)
{
  value = NULL;
  Update();
};

template<class TValue, class TIndex, int DIM, class MAP, class MAPVALUE>
TMDimVar<TValue, TIndex, DIM, MAP, MAPVALUE>::TMDimVar() 
  : TMappedVar<TValue, TIndex, DIM, MAP>()
{
  value = NULL;
  //??Update();
};

//
//.. Update
//
template<class TValue, class TIndex, int DIM, class MAP, class MAPVALUE>
void TMDimVar<TValue, TIndex, DIM, MAP, MAPVALUE>::Update()
{
  int i;
  int level = index.Dim();
  int N = mdim_list->NumSubIndices(level);
  delete [] value;
  value = new MAPVALUE [N];
  for(i = 0; i < N; i++) {
    value[i] = MAPVALUE(mdim_list, index + mdim_list->SubIndex(level, i));
  };
  initialized = true;
};

//
//.. operator=
//
template<class TValue, class TIndex, int DIM, class MAP, class MAPVALUE>
void TMDimVar<TValue, TIndex, DIM, MAP, MAPVALUE>::operator=
(const TMDimVar<TValue, TIndex, DIM, MAP, MAPVALUE> &other)
{
  TMappedVar<TValue, TIndex, DIM, MAP>::operator=(other);
  Update();
};

//
//.. print
//
template<class TValue, class TIndex, int DIM, class MAP, class MAPVALUE>
void TMDimVar<TValue, TIndex, DIM, MAP, MAPVALUE>::print()
{
  int level = index.Dim();
  int N = mdim_list->NumSubIndices(level);
  cout << "[" << index << "->";
  for(int i = 0; i < N; i++) {
    if (i != 0) cout << ",";
    value[i].print();
  };
  cout << "]";
};

#endif







