// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#if !defined(TMDIMVAR_H)
#define TMDIMVAR_H

template<class TValue, class TIndex, int DIM, class MAP, class MAPVALUE> class TMDimVar;

#include <TMDimIndex.hh>
#include <TMDimList.hh>
#include <TMappedVar.hh>

template<class TValue, class TIndex, int DIM, class MAP, class MAPVALUE>
class TMDimVar : public TMappedVar<TValue, TIndex, DIM, MAP>
{
protected:
  /** */ MAPVALUE *value;
public:
  /** */ TMDimVar(TMDimList<TValue, TIndex, DIM, MAP> *a_mdim_list, 
		  TMDimIndex<TIndex> an_index);
  /** */ TMDimVar(const TMDimVar<TValue, TIndex, DIM, MAP, MAPVALUE> &other);
  /** */ TMDimVar();
  /** */ virtual ~TMDimVar() { delete [] value; };
  /** */ virtual void Update();
  /** */ const MAPVALUE& operator[](size_t i) const { 
#ifdef MDEBUG
    if (!initialized) {
      cerr << "trying to use [] on an uninitialized variable" << endl;
      throw InvalidIndex_error(i);
    };      
    if ((i < 0) || (i >= mdim_list->NumSubIndices(index.Dim()))) {
      cerr << "index " << i << "out of bounds" << endl;
      throw InvalidIndex_error(i);
    };
#endif
    return value[i]; 
  };
  /** */ virtual void operator=(const TMDimVar<TValue, TIndex, DIM, MAP, MAPVALUE> &other);
  /** */ virtual void print();
};

#include "TMDimVar.cc"

#endif








