// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#include <CalcGradGreen2D.hh>
#include <CppConstructArg.hh>

template<class POP>
TimeStep<POP>::TimeStep(carg_t arg) : Action(arg)
{
  AddClassName();
  CFLM = ResolveVar<CflManager>(arg);
  old_name = arg.P->ReadString();
  new_name = arg.P->ReadString();
  res_name = arg.P->ReadString();
  partial_step = new TPOperation<TimeStep<POP>::PartialStep, POP>(this);
  copy_field = new TPOperation<TimeStep<POP>::CopyField, POP>(this);
  set_res_to_zero = new TPOperation<TimeStep<POP>::SetResToZero, POP>(this);
  before_step = BuildObject<Action>(arg);
  after_step = BuildObject<Action>(arg);
  PointList *PL = FindObject<PointList>(false, true);
  calc_grad = NULL;
  if (PL->Dim() == 2) {
    calc_grad = new CalcGradGreen2D<POP>(ccarg_t(this) + new_name);
  };
  if (PL->Dim() == 3) {
    Error("sorry 3dim not yet available ...");
  };
  if (calc_grad == NULL) {
    Error("bogus dimension!!!");
  };
};  

template<class POP>
TimeStep<POP>::SubOperator::SubOperator(carg_t arg) : Operator(arg)
{
  AddClassName();
  tstep = FindObject<TimeStep<POP> >(true, true);
  VL = FindObject<VarList>(false, true);
  PL = FindObject<PointList>(false, true);
  var_old = VL->Get2DVar(tstep->OldName());
  var_new = VL->Get2DVar(tstep->NewName());
  var_res = VL->Get2DVar(tstep->ResName());
  CFLM = tstep->Cfl();
};













