// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#ifndef TimeStep_HH
#define TimeStep_HH

template<class POP> class TimeStep;

#include <Action.hh>
#include <carg_t.hh>
#include <Operator.hh>
#include <TPOperation.hh>
#include <CflManager.hh>
#include <VarList.hh>

/** */
template<class POP>
class TimeStep : public Action
{
protected:
  /** */
  struct SubOperator : public Operator {
    CflManager *CFLM;
    TimeStep *tstep;
    size_t ivar;
    VarContainer::var2_t var_old, var_new, var_res;
    SubOperator(carg_t arg);
  };

  /** */ 
  struct PartialStep : public SubOperator {
    PartialStep(carg_t arg) : SubOperator(arg) { AddClassName(); };
    void operate(size_t il, size_t);
  };

  /** */
  struct CopyField : public SubOperator {
    CopyField(carg_t arg) : SubOperator(arg) { AddClassName(); };
    void operate(size_t il, size_t);
  };

  /** */
  struct SetResToZero: public SubOperator {
    SetResToZero(carg_t arg) : SubOperator(arg) { AddClassName(); };
    void operate(size_t il, size_t);
  };

  /** */ Action *copy_field;
  /** */ Action *set_res_to_zero;
  /** */ Action *before_step;
  /** */ Action *partial_step;
  /** */ Action *after_step;
  /** */ Action *calc_grad;
  /** */ string old_name;
  /** */ string new_name;
  /** */ string res_name;
  /** */ CflManager *CFLM;
  /** */ real factor;

  /** */ virtual void CalcGrad() { calc_grad->Act(); };
  /** */ virtual void Copy() { copy_field->Act(); };
  /** */ virtual void SetRes() { set_res_to_zero->Act(); };
  /** */ virtual void Before() { before_step->Act(); };
  /** */ virtual void Partial() { partial_step->Act(); };
  /** */ virtual void After() { after_step->Act(); };

public:
  /** */ TimeStep(carg_t arg);
  /** */ string OldName() { return old_name; };
  /** */ string NewName() { return new_name; };
  /** */ string ResName() { return res_name; };
  /** */ CflManager* Cfl() { return CFLM; };
  /** */ real Factor() { return factor; };
};

template<class POP>
inline void TimeStep<POP>::PartialStep::operate(size_t il, size_t i)
{
  //cout << "i=" << i << " delta_t=" << CFLM->TimeStep(i) << " chl=" << PL->ChDist(i) << endl;
  for (ivar = 0; ivar < VL->NumVars(); ivar++) {
    //cout << var_new[ivar][i] <<endl;
    var_new[ivar][i] = var_old[ivar][i] 
      + tstep->Factor() * CFLM->CFL() * PL->IVol(i) * CFLM->TimeStep(i) * var_res[ivar][i];
  };
};

template<class POP>
inline void TimeStep<POP>::CopyField::operate(size_t il, size_t i)
{
  for (ivar = 0; ivar < VL->NumVars(); ivar++) {
    var_old[ivar][i] = var_new[ivar][i];
  };
};

template<class POP>
inline void TimeStep<POP>::SetResToZero::operate(size_t il, size_t i)
{
  for (ivar = 0; ivar < VL->NumVars(); ivar++) {
    var_res[ivar][i] = 0.0;
  };
};

#include "TimeStep.cc"

#endif















