// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik
// Universitaet Duisburg, Germany
// Institute for Combustion and Gas Dynamics
// University of Duisburg, Germany
// Sun May 23 1999
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#include "VarDiffAdapt2D.hh"

#ifndef VarDiffAdapt2D_CC
#define VarDiffAdapt2D_CC

template <class POP>
VarDiffAdapt2D<POP>::VarDiffAdapt2D(carg_t arg)
  : Mesh2DAction(arg)
{
  AddClassName();
  VL = FindObject<VarList>(false, true);
  POPL = FindObject<POP>(false, true);
  string var_name = arg.P->ReadString();
  gx = VL->Get1DVar("gradx", var_name);
  gy = VL->Get1DVar("grady", var_name);
  R_max = arg.P->ReadReal();
  R_min = arg.P->ReadReal();
  C1 = arg.P->ReadReal();
  C2 = arg.P->ReadReal();
};

template <class POP>
VarDiffAdapt2D<POP>::~VarDiffAdapt2D()
{
};

template <class POP>
void VarDiffAdapt2D<POP>::Act()
{
  real R, gamma, abs_g;
  long int i, j, plink;
  FORALL(i, VL->) {
    abs_g = VABS2(gx[i], gy[i]);
    if (abs_g > 1e-4) {
      abs_g /= MESH->ND(i);
      abs_g -= C1;
      if (abs_g < 0) abs_g = 0;
      R = 1 + abs_g * C2;
      if (R > R_max) R = R_max;
      if (R < R_min) R = R_min;
      gamma = Angle(0, 1, gx[i], gy[i]);
      MESH->SetMatrix(i, 1, R, gamma);
    } else {
      MESH->SetMatrix(i, 1, 1, 1);
    };
  };
};

#endif

