// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#include "vector.hh"

unsigned short int i_reg,default_Dim;
double *dp1_reg,*dp2_reg;

vector::vector()
{
    Dim = default_Dim;
    value = new double[Dim];
};

vector::vector (unsigned short int aDim)
{
    Dim = aDim;
    value = new double[Dim];
};

vector::vector (double v0,double v1)
{
    Dim = 2;
    value = new double[2];
    value[0] = v0;
    value[1] = v1;
};

vector::vector (double v0,double v1,double v2)
{
    Dim = 3;
    value = new double[3];
    value[0] = v0;
    value[1] = v1;
    value[2] = v2;
};

vector::vector (unsigned short int aDim,double *avalue)
{
    Dim = aDim;
    value = avalue;
};

vector::vector (const vector& v)
{
    int i;
    Dim = v.Dim;
    value = new double[Dim];
    for (i=0;i<Dim;i++)
	value[i] = v.value[i];
};

void vector::operator= (const vector& v)
{
    for (i_reg=0;i_reg<Dim;i_reg++)
	value[i_reg] = v[i_reg];
};

double vector::operator* (const vector& v) const
{
    double a=0.0;
    for (i_reg=0;i_reg<Dim;i_reg++) 
	a = a + value[i_reg]*v.value[i_reg];
    return a;
};

vector vector::operator+ (const vector& v) const
{
    vector erg(Dim);
    for (i_reg=0;i_reg<Dim;i_reg++)
	erg.value[i_reg] = value[i_reg] + v.value[i_reg];
    return erg;
};

vector vector::operator- (const vector& v) const
{
    vector erg(Dim);
    for (i_reg=0;i_reg<Dim;i_reg++)
	erg.value[i_reg] = value[i_reg] - v.value[i_reg];
    return erg;
};

vector vector::cross (const vector& v)
{
    vector erg(3);
    erg.value[0] = value[1]*v.value[2] - value[2]*v.value[1];
    erg.value[1] = value[2]*v.value[0] - value[0]*v.value[2];
    erg.value[2] = value[0]*v.value[1] - value[1]*v.value[0];
    return erg;
};

matrix vector::dyad (const vector& v)
{
    int i,j;
    matrix erg(Dim,v.Dim);
    for (i=0;i<Dim;i++) {
	for (j=0;j<v.Dim;j++) {
	    erg[i][j] = value[i]*v.value[j];
	};
    };
    return erg;
};

vector operator* (const vector& v,const double a)
{
    vector erg(v.Dim);
    for (i_reg=0;i_reg<v.Dim;i_reg++)
	erg.value[i_reg] = a*v.value[i_reg];
    return erg;
};

vector operator* (const double a,const vector& v)
{
    vector erg(v.Dim);
    for (i_reg=0;i_reg<v.Dim;i_reg++)
	erg.value[i_reg] = a*v.value[i_reg];
    return erg;
};

ostream& operator<< (ostream& s,const vector v)
{
    int i;
    s << "(";
    for (i=0;i<v.dim();i++) {
	s << v[i];
	if (i != v.dim()-1)
	    s << ",";
    };
    s << ")";
    return s;
};

vector*** vector_array_3 (int I,int J,int K,unsigned short int a_dim)
{
    int i,j;
    vector ***v;
    default_Dim = a_dim;
    v = (vector***) malloc(sizeof(vector***)*I);
    for (i=0;i<I;i++) {
	v[i] = (vector**) malloc(sizeof(vector**)*J);
	for (j=0;j<J;j++) {
	    v[i][j] = new vector[K];
	};
    };
    return v;
};

vector** vector_array_2 (int I,int J,unsigned short int a_dim)
{
    int i;
    vector **v;
    default_Dim = a_dim;
    v = (vector**) malloc(sizeof(vector**)*I);
    for (i=0;i<I;i++) {
	v[i] = new vector[J];
    };
    return v;
};

vector* vector_array_1 (int I,unsigned short int a_dim)
{
  //int i;
  vector *v;
  default_Dim = a_dim;
  v = new vector[I];
  return v;
};



matrix::matrix (unsigned short int aDim1,unsigned short int aDim2)
{
  //int i;
  Dim1 = aDim1;
  Dim2 = aDim2;
  default_Dim = Dim2;
  row = new vector[Dim1];
};

matrix::matrix (const matrix& M)
{
    int i,j;
    Dim1 = M.Dim1;
    Dim2 = M.Dim2;
    row = new vector[Dim1];
    for (i=0;i<Dim1;i++) {
	for (j=0;j<Dim2;j++) {
	    row[i][j] = M[i][j];
	};
    };
};

vector matrix::operator* (const vector& v) const
{
    int i;
    vector erg(v.dim());
//    cout << v.dim() << "\n";
    for (i=0;i<v.dim();i++)
	erg[i] = v*row[i];
    return erg;
};

matrix matrix::operator* (const matrix& A) const
{
    int i,j;
    matrix erg(dim1(),A.dim2());
    for (i=0;i<dim1();i++) 
	for (j=0;j<A.dim2();j++) 
	    erg[i][j] = row[i]*A(j);
    return erg;
};
	    
matrix matrix::operator+ (const matrix& A) const
{
    int i,j;
    matrix erg(Dim1,Dim2);
    for (i=0;i<Dim1;i++)
	for (j=0;j<Dim2;j++) 
	    erg[i][j] = row[i][j] + A[i][j];
    return erg;
};

matrix matrix::operator- (const matrix& A) const
{
    int i,j;
    matrix erg(Dim1,Dim2);
    for (i=0;i<Dim1;i++)
	for (j=0;j<Dim2;j++) 
	    erg[i][j] = row[i][j] - A[i][j];
    return erg;
};

void matrix::make_zero (int r,int c,int dir)
{
    int j;
    double a,b;
    if (fabs(row[r][c]) > 1e-9) {
	a = row[r][c];
	b = row[r+dir][c];
	for (j=0;j<Dim2;j++) {
	    row[r][j] = row[r+dir][j]*a - row[r][j]*b;
	};
    };
};    

matrix matrix::inv ()
{
    int i,j,I;
    I = Dim1;
    matrix M(I,2*I);
    matrix erg(I,I);
    for (i=0;i<I;i++) {
	for (j=0;j<I;j++) {
	    M[i][j] = row[i][j];
	    if (i==j)
		M[i][j+I] = 1;
	    else
		M[i][j+I] = 0;
	};
    };
    for (j=0;j<I-1;j++) {
	for (i=I-1;i>j;i--) {
	    M.make_zero (i,j,-1);
	};
    };
    for (j=I-1;j>0;j--) {
 	for (i=0;i<j;i++) {
 	    M.make_zero (i,j,1);
 	};
    };
    for (i=0;i<I;i++) {
	for (j=0;j<I;j++) {
	    erg[i][j] = M[i][j+I]/M[i][i];
	};
    };
    return erg;
};

matrix matrix::T ()
{
    int i,j;
    matrix erg(Dim2,Dim1);
    for (i=0;i<Dim2;i++)
	for (j=0;j<Dim1;j++)
	    erg[i][j] = row[j][i];
    return erg;
};

vector matrix::operator() (int c) const
{
    int i;
    vector erg(Dim1);
    for (i=0;i<Dim1;i++)
	erg[i] = row[i][c];
    return erg;
};

void matrix::operator= (const matrix& M)
{
    int i,j;
    for (i=0;i<Dim1;i++) {
	for (j=0;j<Dim2;j++) {
	    row[i][j] = M[i][j];
	};
    };
};

matrix operator* (const double a,const matrix& M)
{
    int i,j;
    matrix erg(M.Dim1,M.Dim2);
    for (i=0;i<M.Dim1;i++) {
	for (j=0;j<M.Dim2;j++) {
	    erg[i][j] = a*M[i][j];
	};
    };
    return erg;
};

ostream& operator<< (ostream& s,const matrix M)
{
    int i;
    s << "[";
    for (i=0;i<M.dim1();i++) {
	s << M[i];
	if (i != M.dim1()-1)
	    s << ",";
    };
    s << "]";
    return s;
};














